@@ -14,8 +14,8 @@
  *    Christian Schulte, 1997-1999
  * 
  *  Last change:
- *    $Date: 2002/11/20 12:22:28 $ by $Author: duchier $
- *    $Revision: 1.524.2.4.2.4 $
+ *    $Date: 2004/04/08 15:44:11 $ by $Author: raph $
+ *    $Revision: 1.539 $
  * 
  *  This file is part of Mozart, an implementation 
  *  of Oz 3:
@@ -36,6 +36,7 @@
 #include "board.hh"
 #include "thr_int.hh"
 #include "var_base.hh"
+#include "var_readonly.hh"
 #include "codearea.hh"
 #include "fdomn.hh"
 #include "pickle.hh"
@@ -154,6 +155,7 @@ void AM::init(int argc,char **argv)
   init_cmem();
   ozconf.init();
   osInit();
+  ozstat.init();
   AssRegArray::init();
 
   defaultExceptionHdl = makeTaggedNULL();
@@ -269,7 +271,8 @@ void AM::init(int argc,char **argv)
 
 
 // internal registers
-  statusReg    = (StatusBit)0;
+  statusReg    = (StatusBit) 0;
+  childReady   = NO;
   criticalFlag = NO;
 
   _rootBoard = new Board();
@@ -278,12 +281,12 @@ void AM::init(int argc,char **argv)
   cachedSelf   = NULL;
   gcStep = OddGCStep;
   copyStep = 0;
-  setInEqEq(FALSE);
+  unsetEqEqMode();
   setCurrent(_rootBoard, _rootBoard->getOptVar());
 
   lastThreadID    = 0;
   debugMode       = NO;
-  debugStreamTail = OZ_newVariable();
+  debugPort       = oz_newPort(oz_newReadOnly(_rootBoard));
 
   propLocation    = NO;
 
@@ -314,8 +317,7 @@ void AM::init(int argc,char **argv)
   osSetAlarmTimer(CLOCK_TICK/1000);
 
   //
-  genFreeListManager=new GenFreeListManager();
-  idCounter = new FatInt();
+  initGNameTable();
 
   //
   initSite();
@@ -376,6 +378,10 @@ void AM::init(int argc,char **argv)
 #endif
   taskMinInterval = DEFAULT_MIN_INTERVAL;
 
+  //
+  suspCnt = 0;
+
+  //
   unsetProfileMode();
 }
 
@@ -479,15 +485,30 @@ sigjmp_buf wake_jmp;
 #endif
 volatile int use_wake_jmp = 0;
 
-void AM::suspendEngine(void) {
-
+void AM::suspendEngine(void)
+{
   _rootBoard->install();
 
-  ozstat.printIdle(stdout);
-
   //
   osBlockSignals(OK);
 
+  // erik+kost@ : Before we really attempt to suspend, we've probably
+  //              initiated sending a message(s), but that message(s)
+  //              are probably not yet written out.  'suspendEngine()'
+  //              with its handling of timers and special cases is too
+  //              heavy for figuring that out, so do an eager check:
+  oz_io_check();
+  if (isSetSFlag(IOReady)) {
+    oz_io_handle();
+    if (!threadsPool.isEmpty()) {
+      osUnblockSignals();
+      return;
+    }
+  }
+
+  // still nothing: (attempt to) suspend now;
+  ozstat.printIdle(stdout);
+
   //
   // kost@ : Alarm timer will be reset later (and we don't need to
   // disturb 'select' when we know how long to wait in it).
@@ -517,10 +538,11 @@ void AM::suspendEngine(void) {
     }
 #endif
 
-    unsigned long idle_start = osTotalTime();
-
     //
+    unsigned long idle_start = osTotalTime();
     unsigned int sleepTime = waitTime();
+    //
+    suspCnt++;
 
     // here we set up wake_jmp so that we can longjmp out of a
     // signal handler
@@ -535,6 +557,8 @@ void AM::suspendEngine(void) {
 #endif
       // now perform blocking select
       osBlockSelect(sleepTime);
+      // osBusyWaitSelect(sleepTime);
+
       // it returned normally
       // we block all signals again, but it is ok if we don't make
       // it through because a signal handler is invoked and does
@@ -561,7 +585,7 @@ void AM::suspendEngine(void) {
     // explicit alarm signal is sent while in 'osBlockSelect()');
     handleAlarm(sleepTime);
 
-    ozstat.timeIdle += (osTotalTime() - idle_start);
+    ozstat.timeIdle += sleepTime;
 
     wakeUser();
   }
@@ -622,8 +646,8 @@ void AM::checkStatus(Bool block)
     handleTasks();
 #endif
 
-  if (isSetSFlag(ChildReady)) {
-    unsetSFlag(ChildReady);
+  if (childReady) {
+    childReady = NO;
 #ifdef DENYS_EVENTS
     OZ_eventPush(oz_atom("SIGCHLD"));
 #else
@@ -767,16 +791,16 @@ void handlerPIPE(int)
   //
   // kost@ : let's check for a dead machine;
   // if (isDeadSTDOUT())
-    //am.exitOz(1);
+  //   am.exitOz(1);
   //
-  //prefixError();
-  //message("write on a pipe or other socket with no one to read it ****\n");
+  // prefixError();
+  // message("write on a pipe or other socket with no one to read it ****\n");
 }
 
 void handlerCHLD(int)
 {
   // DebugCheckT(message("a child process' state changed ****\n"));
-  am.setSFlag(ChildReady);
+  am.setChildReady();
   if (use_wake_jmp) {
     use_wake_jmp=0;
     siglongjmp(wake_jmp,1);
@@ -824,12 +848,6 @@ void AM::handleAlarm(int ms)
     }
   }
 
-  if (threadSwitchCounter > 0) {
-    if (--threadSwitchCounter == 0) {
-      setSFlag(ThreadSwitch);
-    }
-  }
-
   if (checkUser())
     setSFlag(UserAlarm);
 
@@ -881,7 +899,9 @@ public:
 void AM::insertUser(int ms, TaggedRef node)
 {
   osBlockSignals();
+
   unsigned int wakeupAt = osTotalTime() + ms;
+
   OzSleep **prev = &sleepQueue;
   for (OzSleep *aux = *prev; aux; prev = &aux->next, aux=aux->next) {
     if (wakeupAt <= aux->time) {
@@ -966,11 +986,13 @@ void AM::wakeUser()
 char flagChar(StatusBit flag)
 {
   switch (flag) {
-  case ThreadSwitch: return 'T';
-  case IOReady:      return 'I';
-  case UserAlarm:    return 'U';
-  case StartGC:      return 'G';
-  default:           return 'X';
+  case TimerInterrupt: return 'R';
+  case IOReady:        return 'I';
+  case UserAlarm:      return 'U';
+  case StartGC:        return 'G';
+  case TaskReady:      return 'T';
+  case SigPending:     return 'S';
+  default:             return 'X';
   }
 }
 #endif
@@ -1073,6 +1095,26 @@ OZ_Return oz_addSuspendVarList3(TaggedRef t1, TaggedRef t2, TaggedRef t3) {
   return SUSPEND;
 }
 
+OZ_Return oz_addSuspendVarList4(TaggedRef t1, TaggedRef t2, TaggedRef t3, TaggedRef t4) {
+  DEREF(t1,t1ptr);
+  Assert(!oz_isRef(t1));
+  if (oz_isVarOrRef(t1))
+    (void) am.addSuspendVarListInline(t1ptr);
+  DEREF(t2,t2ptr);
+  Assert(!oz_isRef(t2));
+  if (oz_isVarOrRef(t2))
+    (void) am.addSuspendVarListInline(t2ptr);
+  DEREF(t3,t3ptr);
+  Assert(!oz_isRef(t3));
+  if (oz_isVarOrRef(t3))
+    (void) am.addSuspendVarListInline(t3ptr);
+  DEREF(t4,t4ptr);
+  Assert(!oz_isRef(t4));
+  if (oz_isVarOrRef(t4))
+    (void) am.addSuspendVarListInline(t4ptr);
+  return SUSPEND;
+}
+
 OZ_Return oz_addSuspendInArgs1(OZ_Term * _OZ_LOC[]) {
   TaggedRef t = OZ_in(0);
   DEREF(t,tptr);
@@ -1112,6 +1154,30 @@ OZ_Return oz_addSuspendInArgs3(OZ_Term * _OZ_LOC[]) {
   return SUSPEND;
 }
 
+OZ_Return oz_addSuspendInArgs4(OZ_Term * _OZ_LOC[]) {
+  TaggedRef t1 = OZ_in(0);
+  DEREF(t1,t1ptr);
+  Assert(!oz_isRef(t1));
+  if (oz_isVarOrRef(t1))
+    (void) am.addSuspendVarListInline(t1ptr);
+  TaggedRef t2 = OZ_in(1);
+  DEREF(t2,t2ptr);
+  Assert(!oz_isRef(t2));
+  if (oz_isVarOrRef(t2))
+    (void) am.addSuspendVarListInline(t2ptr);
+  TaggedRef t3 = OZ_in(2);
+  DEREF(t3,t3ptr);
+  Assert(!oz_isRef(t3));
+  if (oz_isVarOrRef(t3))
+    (void) am.addSuspendVarListInline(t3ptr);
+  TaggedRef t4 = OZ_in(3);
+  DEREF(t4,t4ptr);
+  Assert(!oz_isRef(t4));
+  if (oz_isVarOrRef(t4))
+    (void) am.addSuspendVarListInline(t4ptr);
+  return SUSPEND;
+}
+
 #if OUTLINE_SETEXCEPTIONINFO
 void AM::setExceptionInfo(TaggedRef inf) {
     if (exception.info == NameUnit) {
