@@ -6,8 +6,8 @@
  *    Denys Duchier (1998)
  * 
  *  Last change:
- *    $Date: 2002/03/27 22:25:41 $ by $Author: duchier $
- *    $Revision: 1.43 $
+ *    $Date: 2006/02/20 17:24:03 $ by $Author: mejias $
+ *    $Revision: 1.43.12.1 $
  * 
  *  This file is part of Mozart, an implementation 
  *  of Oz 3:
@@ -452,7 +452,9 @@ OZ_BI_define(BIBitString_make,2,1)
     OZ_Term elt = OZ_head(tail);
     int i;
     if (!OZ_isSmallInt(elt) || (i=OZ_intToC(elt))<0 || i>=w) {
-      oz_typeError(1,"list of small ints");
+      char buffer[100];
+      sprintf(buffer,"list of small ints in range [0,%d)",w);
+      oz_typeError(1,buffer);
     }
     else {
       bs->put(i,OK);
