@@ -11,8 +11,8 @@
  *    Konstantin Popov, 2002
  * 
  *  Last change:
- *    $Date: 2003/12/09 09:09:34 $ by $Author: duchier $
- *    $Revision: 1.40 $
+ *    $Date: 2006/02/20 17:28:34 $ by $Author: mejias $
+ *    $Revision: 1.40.4.2 $
  * 
  *  This file is part of Mozart, an implementation 
  *  of Oz 3:
@@ -214,7 +214,7 @@ void DictHashTable::compactify()
   oldEntries = entries;
 
   // can be zero too:
-  int tableSize = (int) ((double) entries * GDT_IDEALENTRIES);
+  int tableSize = (int) ((double) entries / GDT_IDEALENTRIES);
   Assert(tableSize < oldSize);
   sizeIndex--;
   while (sizeIndex >= 0 && dictHTSizes[sizeIndex] >= tableSize)
@@ -345,7 +345,7 @@ DictNode* DictHashTable::getPairsInArray()
   DictNode* acc = out;
 
   //
-  for (int i = dictHTSizes[sizeIndex], ai = 0; i--; ) {
+  for (int i = dictHTSizes[sizeIndex]; i--; ) {
     DictNode* n = &table[i];
     if (!n->isEmpty()) {
       if (!n->isPointer()) {
@@ -429,6 +429,7 @@ OZ_Term DictHashTable::getArityList(OZ_Term tail)
 	}
       }
     }
+    Assert(ai == entries);
 
     //
     Order_TaggedRef_By_Feat lt;
@@ -480,6 +481,28 @@ OZ_Term DictHashTable::toRecord(OZ_Term lbl)
   }
 }
 
+#if defined(DEBUG_CHECK)
+void DictHashTable::verify()
+{
+  int tableSize = dictHTSizes[sizeIndex];
+  int acc = 0;
+
+  for (int i = tableSize; i--; ) {
+    DictNode *n = &table[i];
+    if (!n->isEmpty()) {
+      if (!n->isPointer()) {
+	acc++;
+      } else {
+	DictNode *sptr = n->getDictNodeSPtr();
+	DictNode *eptr = n->getDictNodeEPtr();
+	acc += eptr - sptr;
+      }
+    }
+  }
+  Assert(acc == entries);
+}
+#endif
+
 //  OZ_Term DictHashTable::htFindOutline(OZ_Term key)
 //  {
 //    return (htFind(key));
@@ -585,7 +608,7 @@ OZ_Return dictionaryGetInline(OZ_Term d, OZ_Term k, OZ_Term &out)
   if (out) {
     return (PROCEED);
   } else {
-    return oz_raise(E_SYSTEM,E_KERNEL,"dict",2,d,k);
+    return oz_raise(E_ERROR,E_KERNEL,"dict",2,d,k);
   }
   Assert(0);
 }
@@ -617,7 +640,7 @@ OZ_Return dictionaryExchangeInline(OZ_Term d, OZ_Term k, OZ_Term value, OZ_Term&
   GetDictAndKey(d,k,dict,key,OK);
   // do not create entry if it does not already exist
   if (dict->exchange(key,value,old,false)) return PROCEED;
-  else return oz_raise(E_SYSTEM,E_KERNEL,"dict",2,d,k);
+  else return oz_raise(E_ERROR,E_KERNEL,"dict",2,d,k);
 }
 
 OZ_DECLAREBI_USEINLINEFUN3(BIdictionaryExchange,dictionaryExchangeInline)
