@@ -14,8 +14,8 @@
  *    Organization or Person (Year(s))
  * 
  *  Last change:
- *    $Date: 2002/12/02 13:28:50 $ by $Author: kornstae $
- *    $Revision: 1.801.2.1.2.5 $
+ *    $Date: 2004/03/24 15:46:10 $ by $Author: bruni $
+ *    $Revision: 1.813 $
  * 
  *  This file is part of Mozart, an implementation 
  *  of Oz 3:
@@ -649,12 +649,6 @@ int engine(Bool init)
 // *** Emulator: execute instructions
 // ------------------------------------------------------------------------
 
- LBLemulate:
-  asmLbl(EMULATE);
-
-  JUMPABSOLUTE( PC );
-
-  asmLbl(END_EMULATE);
 #ifndef THREADED
 LBLdispatcher:
   asmLbl(DISPATCH);
@@ -737,13 +731,16 @@ LBLdispatcher:
   /*
    * Currently unused
    */
-  Case(GETRETURNX) 
-  Case(GETRETURNY) 
-  Case(GETRETURNG) 
-  Case(FUNRETURNX) 
-  Case(FUNRETURNY) 
-  Case(FUNRETURNG) 
-    OZ_error("Impossible.");
+  Case(GETRETURNX) { goto currently_unused; }
+  Case(GETRETURNY) { goto currently_unused; }
+  Case(GETRETURNG) { goto currently_unused; }
+  Case(FUNRETURNX) { goto currently_unused; }
+  Case(FUNRETURNY) { goto currently_unused; }
+  Case(FUNRETURNG) { goto currently_unused; }
+
+  currently_unused:
+  OZ_error("Impossible (opcode should be unused)");
+
   
   Case(CREATEVARIABLEX)
     {
@@ -1403,7 +1400,7 @@ LBLdispatcher:
 
 
   /* a unify void in read case mode */
-Case(GETVOID)
+  Case(GETVOID)
     {
       sPointer += getPosIntArg(PC+1);
       DISPATCH(2);
@@ -2306,24 +2303,24 @@ Case(GETVOID)
 
   Case(TESTLT)
     {
-      TaggedRef A = XPC(1); DEREF0(A,_1);
+      TaggedRef A = XPC(1); DEREF_NONVAR(A);
       
       if (oz_isSmallInt(A)) {
-	TaggedRef B = XPC(2); DEREF0(B,_2);
+	TaggedRef B = XPC(2); DEREF_NONVAR(B);
 	if (oz_isSmallInt(B)) {
 	  LT_IF(smallIntLess(A,B));
 	}
       }
 	
       if (oz_isFloat(A)) {
-	TaggedRef B = XPC(2); DEREF0(B,_2);
+	TaggedRef B = XPC(2); DEREF_NONVAR(B);
 	if (oz_isFloat(B)) {
 	  LT_IF(floatValue(A) < floatValue(B));
 	}
       }
 	
       {
-	TaggedRef B = XPC(2); DEREF0(B,_2);
+	TaggedRef B = XPC(2); DEREF_NONVAR(B);
 	if (oz_isAtom(A) && oz_isAtom(B)) {
 	  LT_IF(strcmp(tagged2Literal(A)->getPrintName(),
 		       tagged2Literal(B)->getPrintName()) < 0);
@@ -2359,8 +2356,8 @@ Case(GETVOID)
 
   Case(TESTLE)
     {
-      TaggedRef A = XPC(1); DEREF0(A,_1);
-      TaggedRef B = XPC(2); DEREF0(B,_2);
+      TaggedRef A = XPC(1); DEREF_NONVAR(A);
+      TaggedRef B = XPC(2); DEREF_NONVAR(B);
       
       if (oz_isSmallInt(A) && oz_isSmallInt(B)) {
 	LT_IF(smallIntLE(A,B));
@@ -2857,6 +2854,9 @@ Case(GETVOID)
 // --- end call/execute -----------------------------------------------------
 // --------------------------------------------------------------------------
 
+#undef EMULATE_NO_CONSCALL
+// for kostja, until we can figure out a better way
+#ifndef EMULATE_NO_CONSCALL
   Case(DECONSCALLX)
     isTailCall = NO; SETAUX(XPC(1)); goto LBLdeconsCall;
   Case(DECONSCALLY)
@@ -2875,8 +2875,7 @@ Case(GETVOID)
       DEREF(taggedPredicate,predPtr);
       if (oz_isConst(taggedPredicate)) {
 	predicate = tagged2Const(taggedPredicate);
-	TypeOfConst typ = predicate->getType();
-	switch (typ) {
+	switch (predicate->getType()) {
 	case Co_Abstraction:
 	  {
 	    Abstraction *def = (Abstraction *) predicate;
@@ -2891,7 +2890,7 @@ Case(GETVOID)
 	      DEREF(taggedArgument,argPtr);
 	      if (oz_isSTuple(taggedArgument)) {
 		SRecord *srec = tagged2SRecord(taggedArgument);
-		int callerArity = srec->getWidth();
+		int callerArity = srec->getTupleWidth();
 		if (srec->getLabel() == AtomPair &&
 		    callerArity == calleeArity - 1) {
 		  int i = callerArity;
@@ -2914,30 +2913,26 @@ Case(GETVOID)
 	case Co_Builtin:
 	  {
 	    Builtin *bi = (Builtin *) predicate;
-	    int calleeArity = bi->getArity();
-	    if (calleeArity == 2) {   // arity is correct
+	    predArity = bi->getArity();
+	    if (predArity == 2) {   // arity is correct
 	      if (!isTailCall) PC = PC+2;
-	      predArity = 2;
 	      goto LBLcall;
 	    } else {   // deconstruct
 	      TaggedRef taggedArgument = XREGS[0];
 	      DEREF(taggedArgument,argPtr);
+	      Assert(!oz_isRef(taggedArgument));
 	      if (oz_isSTuple(taggedArgument)) {
 		SRecord *srec = tagged2SRecord(taggedArgument);
-		int callerArity = srec->getWidth();
+		int callerArity = srec->getTupleWidth();
 		if (srec->getLabel() == AtomPair &&
-		    callerArity == calleeArity - 1) {
-		  int i = callerArity;
-		  XREGS[i] = XREGS[1];
-		  while (--i >= 0)
-		    XREGS[i] = srec->getArg(i);
+		    callerArity == predArity - 1) {
+		  XREGS[callerArity] = XREGS[1];
+		  while (--callerArity >= 0)
+		    XREGS[callerArity] = srec->getArg(callerArity);
 		  if (!isTailCall) PC = PC+2;
-		  predArity = calleeArity;
 		  goto LBLcall;
 		}
-	      }
-	      Assert(!oz_isRef(taggedArgument));
-	      if (oz_isVarOrRef(taggedArgument)) {
+	      } else if (oz_isVarOrRef(taggedArgument)) {
 		SUSP_PC(argPtr,PC);
 	      }
 	      RAISE_ARITY(taggedPredicate,OZ_toList(2,XREGS));
@@ -2953,7 +2948,7 @@ Case(GETVOID)
       }
       Assert(!oz_isRef(taggedPredicate));
       if (oz_isVarOrRef(taggedPredicate)) {
-	SUSP_PC(predPtr,PC);
+  	SUSP_PC(predPtr,PC);
       }
       RAISE_APPLY(taggedPredicate,OZ_toList(2,XREGS));
     }
@@ -3047,6 +3042,24 @@ Case(GETVOID)
       }
       RAISE_APPLY(taggedPredicate,OZ_toList(callerArity,XREGS));
     }
+#else
+    Case(DECONSCALLX)     { goto no_alicecalls; }
+    Case(DECONSCALLY)     { goto no_alicecalls; }
+    Case(DECONSCALLG)     { goto no_alicecalls; }
+    Case(TAILDECONSCALLX) { goto no_alicecalls; }
+    Case(TAILDECONSCALLG) { goto no_alicecalls; }
+    Case(CONSCALLX)       { goto no_alicecalls; }
+    Case(CONSCALLY)       { goto no_alicecalls; }
+    Case(CONSCALLG)       { goto no_alicecalls; }
+    Case(TAILCONSCALLX)   { goto no_alicecalls; }
+    Case(TAILCONSCALLG)   { goto no_alicecalls; }
+
+  no_alicecalls:
+    {
+      Assert(0);
+      return T_ERROR;
+    }
+#endif
 
 // -------------------------------------------------------------------------
 // INSTRUCTIONS: MISC: ERROR/NOOP/default
@@ -3599,8 +3612,10 @@ ThreadReturn debugExit(ProgramCounter PC, RefsArray * Y, Abstraction * CAP) {
 	if (dbg->arguments[iarity] != NameVoidRegister)
 	  for (int i = oarity; i--; ) {
 	    TaggedRef x = loc->getOutValue(bi,i);
-	    if (OZ_unify(dbg->arguments[iarity + i], x) == FAILED)
+	    if (OZ_unify(dbg->arguments[iarity + i], x) == FAILED) {
+	      //--** should be a HF_TELL(dbg->arguments[iarity + i], x);
 	      return T_FAILURE;
+	    }
 	  }
 	else
 	  for (int i = oarity; i--; )
