@@ -11,8 +11,8 @@
  *    Michael Mehl (1997)
  * 
  *  Last change:
- *    $Date: 2002/10/23 14:11:43 $ by $Author: duchier $
- *    $Revision: 1.293.2.1.2.1 $
+ *    $Date: 2004/05/19 19:37:16 $ by $Author: duchier $
+ *    $Revision: 1.295.8.1 $
  * 
  *  This file is part of Mozart, an implementation 
  *  of Oz 3:
@@ -447,7 +447,11 @@ long OZ_intToCL(OZ_Term term)
 
 unsigned long OZ_intToCulong(OZ_Term term)
 {
-  return tagged2BigInt(oz_deref(term))->getUnsignedLong();
+  term = oz_deref(term);
+  if (oz_isSmallInt(term))
+    return (unsigned long) tagged2SmallInt(term);
+  else
+    return tagged2BigInt(term)->getUnsignedLong();
 }
 
 OZ_Term OZ_CStringToInt(char *str)
