@@ -6,8 +6,8 @@
  *    Christian Schulte, 1999
  * 
  *  Last change:
- *    $Date: 2004/04/08 15:44:11 $ by $Author: raph $
- *    $Revision: 1.52 $
+ *    $Date: 2006/05/15 12:02:31 $ by $Author: mejias $
+ *    $Revision: 1.52.2.1 $
  * 
  *  This file is part of Mozart, an implementation 
  *  of Oz 3:
@@ -1022,9 +1022,9 @@ void AM::doGCollect(void) {
   gCollect(ozconf.gcVerbosity);
 
   /* calc limits for next gc */
-  int used   = getUsedMemory();
-  int free   = min(ozconf.heapFree,99);
-  int wanted = max(((long) used) * (100 / (100 - free)),
+  unsigned int used   = getUsedMemory();
+  unsigned int free   = min(ozconf.heapFree,99);
+  unsigned int wanted = max(((long) used) * (100 / (100 - free)),
 		   ozconf.heapMinSize);
 
   /* Try to align as much as possible to end of blocksize */
