@@ -11,8 +11,8 @@
  *    Per Brand, 1998
  * 
  *  Last change:
- *    $Date: 2002/11/24 23:12:04 $ by $Author: duchier $
- *    $Revision: 1.121.4.2 $
+ *    $Date: 2003/04/28 12:32:28 $ by $Author: glynn $
+ *    $Revision: 1.132 $
  * 
  *  This file is part of Mozart, an implementation 
  *  of Oz 3:
@@ -43,19 +43,18 @@
 
 #include "var_obj.hh"
 #include "var_class.hh"
-#include "chain.hh"
+#include "chain.hh" 
 #include "state.hh"
 #include "fail.hh"
 #include "interFault.hh"
-#include "protocolCredit.hh"
 #include "port.hh"
 #include "dpResource.hh"
 #include "protocolState.hh"
 #include "protocolFail.hh"
 #include "msgContainer.hh"
 #include "dpMarshaler.hh"
-#include "flowControl.hh"
 #include "ozconfig.hh"
+#include "memaux.hh"
 
 #include "os.hh"
 #include "connection.hh"
@@ -73,10 +72,48 @@ int  globalOSWriteCounter = 0;
 int  globalOSReadCounter = 0;
 int  globalContCounter = 0;
 
+int  btResize = 0;
+int  btCompactify = 0;
+int  otResize = 0;
+int  otCompactify = 0;
+
 OZ_Term defaultConnectionProcedure = 0;
 
 FILE *logfile=stdout;
 
+int   sendJobbCntr = 0;
+SendJobb  *sendJobbFirst = NULL;
+SendJobb  *sendJobbLast = NULL;
+
+
+
+OZ_Return newSendJobb(int id){
+  ControlVarNew(controlvar,oz_rootBoard());
+  if (sendJobbFirst == NULL && sendJobbLast == NULL)
+    {
+      sendJobbFirst = new SendJobb(id, controlvar,NULL);
+      sendJobbLast  = sendJobbFirst; 
+    }
+  else
+    {
+      sendJobbFirst->next = new SendJobb(id, controlvar,NULL);
+      sendJobbFirst = sendJobbFirst->next;
+    }
+  return suspendOnControlVar();
+}
+
+void ResumeSendJobb(int id){
+  SendJobb **tmp = &sendJobbLast;
+  while(*tmp != NULL && (*tmp)->id != id)tmp=&((*tmp)->next);
+  if (*tmp != NULL){
+    ControlVarResume((*tmp)->cntrlVar);
+    SendJobb *d = (*tmp)->next;
+    delete(*tmp);
+    *tmp = d;
+    if(d == NULL) sendJobbFirst = (*tmp);
+  }
+}
+
 /* *********************************************************************/
 /*   init;                                                             */
 /* *********************************************************************/
@@ -106,10 +143,8 @@ static void initGateStream()
   {
     Tertiary *t=(Tertiary*)new PortWithStream(oz_currentBoard(),GateStream);
     globalizeTert(t);
-    int ind = t->getIndex();
-    Assert(ind==0);
-    OwnerEntry* oe=OT->getOwner(ind);
-    oe->setUp(ind);
+    OB_TIndex ind = MakeOB_TIndex(t->getTertPointer());
+    OwnerEntry* oe = ownerIndex2ownerEntry(ind);
     oe->makePersistent();
   }
 }
@@ -129,7 +164,6 @@ static void initDPCore();
 
 void initDP()
 {
-  //cerr << "sizeof(PortProxy) = " << sizeof(PortProxy) << endl;
   //
   if (perdioInitialized)
     return;
@@ -150,12 +184,10 @@ void initDP()
 /*   Utility routines                                      */
 /* *********************************************************************/
 
-//
-void send(MsgContainer *msgC, int priority)
+void send(MsgContainer *msgC)
 {
-  globalSendCounter++;
 
-  int ret=msgC->getDestination()->send(msgC, priority);
+  int ret=msgC->getDestination()->send(msgC);
 
   switch (ret) {
   case PERM_NOT_SENT:
@@ -169,7 +201,7 @@ void send(MsgContainer *msgC, int priority)
 }
 
 DSite* getSiteFromTertiaryProxy(Tertiary* t){
-  BorrowEntry *be=BT->getBorrow(t->getIndex());
+  BorrowEntry *be = borrowIndex2borrowEntry(MakeOB_TIndex(t->getTertPointer()));
   Assert(be!=NULL);
   return be->getNetAddress()->site;}
 
@@ -209,7 +241,9 @@ void pendThreadAddMoveToEnd(PendThread **pt){
   *pt=new PendThread(NULL,NULL,MOVEEX);
 }
 
-void pendThreadAddRAToEnd(PendThread **pt,DSite *s1, DSite *s2,int index){
+void
+pendThreadAddRAToEnd(PendThread **pt,DSite *s1, DSite *s2,Ext_OB_TIndex index)
+{
   while(*pt!=NULL){pt= &((*pt)->next);}
   *pt=new PendThread(NULL,NULL,(TaggedRef)s1,(TaggedRef)s2,
 		     (TaggedRef)index,REMOTEACCESS);
@@ -227,8 +261,8 @@ void gcBorrowTableUnusedFramesImpl() {
 }
 
 void gcProxyRecurseImpl(Tertiary *t) {
-  int i = t->getIndex();
-  BorrowEntry *be=BT->getBorrow(i);
+  OB_TIndex i = MakeOB_TIndex(t->getTertPointer());
+  BorrowEntry *be=borrowIndex2borrowEntry(i);
   if(be->isGCMarked()){
     PD((GC,"borrow already marked:%d",i));
     return;}
@@ -240,8 +274,8 @@ void gcProxyRecurseImpl(Tertiary *t) {
 
 void gcManagerRecurseImpl(Tertiary *t) {
   Assert(!t->isFrame());
-  int i = t->getIndex();
-  OwnerEntry *oe=OT->getOwner(i);
+  OB_TIndex i = MakeOB_TIndex(t->getTertPointer());
+  OwnerEntry *oe=ownerIndex2ownerEntry(i);
   if(oe->isGCMarked()){
     PD((GC,"owner already marked:%d",i));
     return;
@@ -291,8 +325,12 @@ void gcPerdioRootsImpl()
     OT->gcOwnerTableRoots();
     BT->gcBorrowTableRoots();
     gcGlobalWatcher();
-    flowControler->gcEntries();
     gcDeferEvents();
+    SendJobb* tmp=sendJobbLast;
+    while(tmp){
+      oz_gCollectTerm(tmp->cntrlVar,tmp->cntrlVar);
+      tmp = tmp->next;
+    }
     // marshalers are not collected here, but through allocated
     // continuations ('MsgContainer::gcMsgC()');
   }
@@ -304,7 +342,7 @@ void gcPerdioFinalImpl()
   if (isPerdioInitializedImpl()) {
     BT->gcBorrowTableFinal();
     OT->gcOwnerTableFinal(); 
-    RHT->gcResourceTable();
+    RHT->gcResourceTable();	// after fixing the owner table;
     gcDSiteTable();
     comController_finishGCComObjs();
   }
@@ -352,50 +390,39 @@ void cellifyObject(Object* o){
 void globalizeTert(Tertiary *t)
 { 
   Assert(t->isLocal());
-
+  
+  // The state and the lock of the object must 
+  // be transfered into a distributable form.
+  
+  OwnerEntry *oe_manager;
+  OB_TIndex manI = ownerTable->newOwner(oe_manager);
+  oe_manager->mkTertiary(t);
+  
   switch(t->getType()) {
   case Co_Cell:
     {
-      OwnerEntry *oe_manager;
-      int manI=ownerTable->newOwner(oe_manager);
-      PD((GLOBALIZING,"GLOBALIZING cell index:%d",manI));
-      oe_manager->mkTertiary(t);
       globalizeCell((CellLocal*)t,manI);
       return;
     }
   case Co_Lock:
     {
-      OwnerEntry *oe_manager;
-      int manI=ownerTable->newOwner(oe_manager);
-      PD((GLOBALIZING,"GLOBALIZING lock index:%d",manI));
-      oe_manager->mkTertiary(t);
       globalizeLock((LockLocal*)t,manI);
       return;
     }
   case Co_Object:
-    {
-      cellifyObject((Object*) t);
-      break;
-    }
+    cellifyObject((Object*) t);
   case Co_Port:
-    break;
+    {
+      t->setTertType(Te_Manager);
+      t->setTertPointer(OB_TIndex2Ptr(manI));
+      return; 
+    }
   default:
-    Assert(0);
+    {
+      OZ_error("Globalization of unknown tert type %d\n",t->getType());
+      Assert(0);
+    }
   }
-
-  t->setTertType(Te_Manager);
-  OwnerEntry *oe;
-  int i = ownerTable->newOwner(oe);
-  PD((GLOBALIZING,"GLOBALIZING port/object index:%d",i));
-  DebugCode(if(t->getType()==Co_Object)
-  {
-    PD((SPECIAL,"object:%x class%x",t,((Object *)t)->getClass()));})
-  
-  oe->mkTertiary(t);
-  t->setIndex(i);
-  //    DebugCode(if(t->getType()==Co_Object) {
-  //      PD((SPECIAL,"object:%x class%x",t,((Object *)t)->getClass()));
-  //    })
 }
 
 
@@ -460,181 +487,147 @@ Bool localizeTertiary(Tertiary*t){
 /*  Main Receive                                      */
 /**********************************************************************/
 
-OwnerEntry* maybeReceiveAtOwner(DSite* mS,int OTI){
-  if(mS==myDSite){
-    OwnerEntry *oe=OT->getOwner(OTI);
-    Assert(!oe->isFree());
-    return oe;
-  }
-  return NULL;
+OwnerEntry* maybeReceiveAtOwner(DSite* mS, Ext_OB_TIndex extOTI)
+{
+  return ((mS == myDSite) ? OT->extOTI2ownerEntry(extOTI) : (OwnerEntry*) 0);
 }
 
-inline OwnerEntry* receiveAtOwner(int OTI){
-  OwnerEntry *oe=OT->getOwner(OTI);
-  Assert(!oe->isFree());
-  return oe;
+static inline OwnerEntry* receiveAtOwner(Ext_OB_TIndex extOTI)
+{
+  return (OT->extOTI2ownerEntry(extOTI));
 }
 
-BorrowEntry* receiveAtBorrow(DSite* mS,int OTI){
-  NetAddress na=NetAddress(mS,OTI);
-  BorrowEntry* be=BT->find(&na);
-  Assert(be!=NULL);
-  return be;
+BorrowEntry* receiveAtBorrow(DSite* mS, Ext_OB_TIndex extOTI)
+{
+  return (BT->find(extOTI, mS));
 }
 
-inline BorrowEntry* maybeReceiveAtBorrow(DSite* mS,int OTI){
-  NetAddress na=NetAddress(mS,OTI);
-  BorrowEntry* be=BT->find(&na);
-
-  return be;
+inline BorrowEntry* maybeReceiveAtBorrow(DSite* mS, Ext_OB_TIndex extOTI)
+{
+  return ((mS != myDSite) ? BT->find(extOTI, mS) : (BorrowEntry *) 0);
 }
 
-void msgReceived(MsgContainer* msgC)
+void msgReceived(MsgContainer* msgC,DSite *sender)
 {
   Assert(oz_onToplevel());
-
-  globalRecCounter++;
+  DebugCode(OT->checkEntries());
 
   MessageType mt = msgC->getMessageType();
 
   PD((MSG_RECEIVED,"msg type %d",mt));
-
+  //    if (mt != M_PING)
+  //      fprintf(stdout, "msg received %d pid:%d\n ",
+  //  	    mt, myDSite->getTimeStamp()->pid);
+  fflush(stdout);
   switch (mt) {
   case M_PORT_SEND:   
     {
-      int portIndex;
+      Ext_OB_TIndex portIndex;
       OZ_Term t;
       msgC->get_M_PORT_SEND(portIndex,t);
       OwnerEntry *oe=receiveAtOwner(portIndex);
-      Assert(oe);
-      PortManager *pm=(PortManager*)(oe->getTertiary());
-      Assert(pm->checkTertiary(Co_Port,Te_Manager));
-
-      doPortSend(pm,t,NULL);
-
+      if (oe)
+	{
+	  PortManager *pm=(PortManager*)(oe->getTertiary());
+	  Assert(pm->checkTertiary(Co_Port,Te_Manager));
+	  doPortSend(pm,t,NULL);
+	}
       break;
     }
-  case M_ASK_FOR_CREDIT:
+  case M_UPDATE_REFERENCE:
     {
-      int na_index;
-      DSite* rsite;
-      msgC->get_M_ASK_FOR_CREDIT(na_index,rsite);
-      PD((MSG_RECEIVED,"ASK_FOR_CREDIT index:%d site:%s",
-	  na_index,rsite->stringrep()));
-      OwnerEntry *oe=receiveAtOwner(na_index);
-      Credit c= oe->getCreditBig();
-
-      MsgContainer *newmsgC = msgContainerManager->newMsgContainer(rsite);
-      Assert(c.owner==NULL);
-      newmsgC->put_M_BORROW_CREDIT(myDSite,na_index,c.credit);
-
-      send(newmsgC,-1);
+      Ext_OB_TIndex index;
+      msgC->get_M_UPDATE_REFERENCE(index);
+      OwnerEntry *oe=receiveAtOwner(index);
+      if (oe)
+	oe->updateReference(sender);
+      else
+	{
+	  // If the owner entry does not exist the entity 
+	  // has been reclaimed. A stub entry will now be created and
+	  // sent back instead.
+	  MsgContainer *msgC = msgContainerManager->newMsgContainer(sender);
+	  msgC->put_M_BORROW_REF(createFailedEntity(index,FALSE));
+	  send(msgC);
+	}
       break;
     }
+  case M_OWNER_REF: 
+   {
+     int val1, val2, type;
+     Ext_OB_TIndex index;
+     msgC->get_M_OWNER_REF(index,type,val1,val2);
 
-  case M_OWNER_CREDIT: 
+     PD((MSG_RECEIVED,"OWNER_REF index:%d type:%d val1:%d val2:%d",index,type,val1,val2));
+     OwnerEntry *oe=receiveAtOwner(index);
+     if (oe)
+       {
+	 oe->mergeReference(CreateRRinstance(type,val1,val2));
+       }
+     break;
+   }
+   
+  case M_BORROW_REF: 
     {
-      int index;
-      int cint;
-      Credit c;
-      msgC->get_M_OWNER_CREDIT(index,cint);
-      c.owner=NULL;
-      c.credit=cint;
-
-      PD((MSG_RECEIVED,"OWNER_CREDIT index:%d credit:%d",index,c));
-      receiveAtOwner(index)->addCredit(c);
-      break;
-    }
-
-  case M_OWNER_SEC_CREDIT:
-    {
-      int index;
-      int cint;
-      Credit c;
-      DSite* s;
-      msgC->get_M_OWNER_SEC_CREDIT(s,index,cint);
-      c.owner=myDSite; // I am the owner of this secondary credit
-      c.credit=cint;
-      PD((MSG_RECEIVED,"OWNER_SEC_CREDIT site:%s index:%d credit:%d",
-	  s->stringrep(),index,c));    
-//        printf("received M_OWNER_SEC_CREDIT %x %d %d %x\n",
-//  	     (int)s,index,c.credit,(int)c.owner);
-
-      receiveAtBorrow(s,index)->addCredit(c);
-      break;
-    }
-
-  case M_BORROW_CREDIT:  
-    {
-      int si;
-      int cint;
-      Credit c;
-      DSite* sd;
-      msgC->get_M_BORROW_CREDIT(sd,si,cint);
-      c.owner=NULL;
-      c.credit=cint;
-      PD((MSG_RECEIVED,"BORROW_CREDIT site:%s index:%d credit:%d",
-	  sd->stringrep(),si,c));
-      //The entry might have been gc'ed. If so send the 
-      //credit back. 
-      //erik 
-      NetAddress na=NetAddress(sd,si);
-      BorrowEntry* be=BT->find(&na);
-      if(be==NULL){
-	sendCreditBack(na.site,na.index,c);}
-      else {
-	be->addCredit(c);}
+      OZ_Term dummy;
+      msgC->get_M_BORROW_REF(dummy);
       break;
     }
 
   case M_REGISTER:
     {
-      int OTI;
-      DSite* rsite;
-      msgC->get_M_REGISTER(OTI,rsite);
-      PD((MSG_RECEIVED,"REGISTER index:%d site:%s",OTI,rsite->stringrep()));
-      OwnerEntry *oe=receiveAtOwner(OTI);
-      if (oe->isVar()) {
-	(GET_VAR(oe,Manager))->registerSite(rsite);
-      } else {
-	sendRedirect(rsite,OTI,OT->getOwner(OTI)->getRef());
-      }
+      Ext_OB_TIndex extOTI;
+      msgC->get_M_REGISTER(extOTI);
+      PD((MSG_RECEIVED,"REGISTER index:%d site:%s",extOTI,sender->stringrep()));
+      OwnerEntry *oe=receiveAtOwner(extOTI);
+      if (oe)
+	{
+	  if (oe->isVar()) {
+	    (GET_VAR(oe,Manager))->registerSite(sender);
+	  } else {
+	    sendRedirect(sender,MakeOB_TIndex(oe),oe->getRef());
+	  }
+	}
       break;
     }
 
   case M_DEREGISTER:
     {
-      int OTI;
-      DSite* rsite;
-      msgC->get_M_REGISTER(OTI,rsite);
-      PD((MSG_RECEIVED,"REGISTER index:%d site:%s",OTI,rsite->stringrep()));
-      OwnerEntry *oe=receiveAtOwner(OTI);
-      if (oe->isVar()) {
-	(GET_VAR(oe,Manager))->deregisterSite(rsite);
-      } else {
-	if(USE_ALT_VAR_PROTOCOL){
-	  recDeregister(OT->getOwner(OTI)->getRef(),rsite);}
-      }
+      Ext_OB_TIndex extOTI;
+      msgC->get_M_DEREGISTER(extOTI);
+      PD((MSG_RECEIVED,"REGISTER index:%d site:%s",extOTI,sender->stringrep()));
+      OwnerEntry *oe=receiveAtOwner(extOTI);
+      if (oe)
+	{
+	  if (oe->isVar()) {
+	    (GET_VAR(oe,Manager))->deregisterSite(sender);
+	  } else {
+	    if(USE_ALT_VAR_PROTOCOL){
+	      recDeregister(oe->getRef(),sender);}
+	  }
+	}
       break;
     }
 
   case M_GET_LAZY:
     {
-      int OTI;
+      Ext_OB_TIndex extOTI;
       DSite* rsite;
       int lazyFlag;
       //
-      msgC->get_M_GET_LAZY(OTI, lazyFlag, rsite);
+      msgC->get_M_GET_LAZY(extOTI, lazyFlag, rsite);
       PD((MSG_RECEIVED,"M_GET_LAZY index:%d site:%s",
-	  OTI, rsite->stringrep()));
+	  extOTI, rsite->stringrep()));
       //
-      OwnerEntry *oe = receiveAtOwner(OTI);
+      OwnerEntry *oe = receiveAtOwner(extOTI);
       //
-      OZ_Term t = oe->getTertTerm();
+      if (oe)
+	{
+	  OZ_Term t = oe->getTertTerm();
 
-      //
-      switch (lazyFlag) {
-      case OBJECT_AND_CLASS:
+	  //
+	  switch (lazyFlag) {
+	  case OBJECT_AND_CLASS:
 	{
 	  Assert(oz_isObject(t));
 	  Object *o = (Object *) tagged2Const(t);
@@ -646,9 +639,9 @@ void msgReceived(MsgContainer* msgC)
 	  MsgContainer *msgC = msgContainerManager->newMsgContainer(rsite);
 
 	  //
-	  msgC->put_M_SEND_LAZY(myDSite, OTI, OBJECT_AND_CLASS,
+	  msgC->put_M_SEND_LAZY(myDSite, extOTI, OBJECT_AND_CLASS,
 				o->getClassTerm());
-	  send(msgC, -1);
+	  send(msgC);
 	}
 	// no break here! - proceed with the 'OBJECT' case;
 
@@ -659,8 +652,8 @@ void msgReceived(MsgContainer* msgC)
 	  MsgContainer *msgC = msgContainerManager->newMsgContainer(rsite);
 
 	  //
-	  msgC->put_M_SEND_LAZY(myDSite, OTI, OBJECT, t);
-	  send(msgC, -1);
+	  msgC->put_M_SEND_LAZY(myDSite, extOTI, OBJECT, t);
+	  send(msgC);
 	}	
 	break;
 
@@ -668,7 +661,7 @@ void msgReceived(MsgContainer* msgC)
 	OZ_error("undefined/unimplemented lazy protocol!");
 	break;
       }
-
+	}
       //
       break;
     }
@@ -676,7 +669,7 @@ void msgReceived(MsgContainer* msgC)
   case M_SEND_LAZY:
     {
       DSite* sd;
-      int si;
+      Ext_OB_TIndex si;
       int lazyFlag;
       OZ_Term t;
 
@@ -735,7 +728,7 @@ void msgReceived(MsgContainer* msgC)
   case M_REDIRECT:
     {
       DSite* sd;
-      int si;
+      Ext_OB_TIndex si;
       TaggedRef val;
       msgC->get_M_REDIRECT(sd,si,val);
       PD((MSG_RECEIVED,"M_REDIRECT site:%s index:%d val%s",
@@ -753,48 +746,49 @@ void msgReceived(MsgContainer* msgC)
 
   case M_SURRENDER:
     {
-      int OTI;
-      DSite* rsite;
+      Ext_OB_TIndex extOTI;
       TaggedRef v;
-      msgC->get_M_SURRENDER(OTI,rsite,v);
+      msgC->get_M_SURRENDER(extOTI,v);
       PD((MSG_RECEIVED,"M_SURRENDER index:%d site:%s val%s",
-	  OTI,rsite->stringrep(),toC(v)));
-      OwnerEntry *oe = receiveAtOwner(OTI);
-
-      if (oe->isVar()) {
-	PD((PD_VAR,"SURRENDER do it"));
-	GET_VAR(oe,Manager)->surrender(oe->getPtr(),v,rsite);
-      } else {
-	PD((PD_VAR,"SURRENDER discard"));
-	PD((WEIRD,"SURRENDER discard"));
-	// ignore redirect: NOTE: v is handled by the usual garbage collection
+	  extOTI,sender->stringrep(),toC(v)));
+      OwnerEntry *oe = receiveAtOwner(extOTI);
+      if (oe){
+	if (oe->isVar()) {
+	  PD((PD_VAR,"SURRENDER do it"));
+	  GET_VAR(oe,Manager)->surrender(oe->getPtr(),v,sender);
+	} else {
+	  PD((PD_VAR,"SURRENDER discard"));
+	  PD((WEIRD,"SURRENDER discard"));
+	  // ignore redirect: NOTE: v is handled by the usual garbage collection
+	}
       }
+            
       break;
     }
 
   case M_GETSTATUS:
     {
-      DSite* site;
-      int OTI;
-      msgC->get_M_GETSTATUS(site,OTI);
-      PD((MSG_RECEIVED,"M_GETSTATUS index:%d",OTI));
-      OwnerEntry *oe = receiveAtOwner(OTI);
-
-      if(oe->isVar()){
-	varGetStatus(site,OTI,oz_status(oe->getValue()));}
+      Ext_OB_TIndex extOTI;
+      msgC->get_M_GETSTATUS(extOTI);
+      PD((MSG_RECEIVED,"M_GETSTATUS index:%d",extOTI));
+      OwnerEntry *oe = receiveAtOwner(extOTI);
+      if (oe)
+	{
+	  if(oe->isVar()){
+	    varGetStatus(sender,extOTI,oz_status(oe->getValue()));}
+	}
       break;
     }
 
   case M_SENDSTATUS:
     {
       DSite* site;
-      int OTI;
+      Ext_OB_TIndex extOTI;
       TaggedRef status;
-      msgC->get_M_SENDSTATUS(site,OTI,status);
+      msgC->get_M_SENDSTATUS(site,extOTI,status);
       PD((MSG_RECEIVED,"M_SENDSTATUS site:%s index:%d status:%d",
-      	  site->stringrep(),OTI,status));
-      NetAddress na=NetAddress(site,OTI);
-      BorrowEntry *be=BT->find(&na);
+      	  site->stringrep(),extOTI,status));
+      BorrowEntry *be = BT->find(extOTI, site);
       if(be==NULL){
 	PD((WEIRD,"receive M_SENDSTATUS after gc"));
 	break;}
@@ -805,71 +799,74 @@ void msgReceived(MsgContainer* msgC)
 
   case M_ACKNOWLEDGE:
     {
-      DSite* sd;
-      int si;
-      msgC->get_M_ACKNOWLEDGE(sd,si);
-      PD((MSG_RECEIVED,"M_ACKNOWLEDGE site:%s index:%d",sd->stringrep(),si));
-
-      NetAddress na=NetAddress(sd,si);
-      BorrowEntry *be=BT->find(&na);
+      Ext_OB_TIndex si;
+      msgC->get_M_ACKNOWLEDGE(si);
+      PD((MSG_RECEIVED,"M_ACKNOWLEDGE site:%s index:%d",sender->stringrep(),si));
+      BorrowEntry *be = BT->find(si, sender);
       if (be) {
 	Assert(be->isVar());
 	GET_VAR(be,Proxy)->acknowledge(be->getPtr(), be);
       }
-
       break;
     }
   case M_CELL_LOCK_GET:
     {
-      int OTI;
+      Ext_OB_TIndex extOTI;
       DSite* rsite;
-      msgC->get_M_CELL_LOCK_GET(OTI,rsite);
-      PD((MSG_RECEIVED,"M_CELL_LOCK_GET index:%d site:%s",OTI,rsite->stringrep()));
-      cellLockReceiveGet(receiveAtOwner(OTI),rsite);
+      msgC->get_M_CELL_LOCK_GET(extOTI,rsite);
+      PD((MSG_RECEIVED,"M_CELL_LOCK_GET index:%d site:%s",extOTI,rsite->stringrep()));
+      OwnerEntry *oe=receiveAtOwner(extOTI);
+      if (oe)
+	{
+	  cellLockReceiveGet(oe,rsite);
+	}
       break;
     }
    case M_CELL_CONTENTS:
     {
       DSite* rsite;
-      int OTI;
+      Ext_OB_TIndex extOTI;
       TaggedRef val;
-      msgC->get_M_CELL_CONTENTS(rsite,OTI,val);
+      msgC->get_M_CELL_CONTENTS(rsite,extOTI,val);
       PD((MSG_RECEIVED,"M_CELL_CONTENTS index:%d site:%s val:%s",
-	  OTI,rsite->stringrep(),toC(val)));
-
-      OwnerEntry* oe=maybeReceiveAtOwner(rsite,OTI);
+	  extOTI,rsite->stringrep(),toC(val)));
+      
+      OwnerEntry* oe=maybeReceiveAtOwner(rsite,extOTI);
       if(oe!=NULL){
-	cellReceiveContentsManager(oe,val,OTI);
+	cellReceiveContentsManager(oe,val,extOTI);
 	break;}
-
-      cellReceiveContentsFrame(receiveAtBorrow(rsite,OTI),val,rsite,OTI);
+      cellReceiveContentsFrame(receiveAtBorrow(rsite, extOTI), val, rsite, extOTI);
       break;
     }
   case M_CELL_READ:
     {
-      int OTI;
+      Ext_OB_TIndex extOTI;
       DSite* fS;
-      msgC->get_M_CELL_READ(OTI,fS);
+      msgC->get_M_CELL_READ(extOTI,fS);
       PD((MSG_RECEIVED,"M_CELL_READ"));
-      cellReceiveRead(receiveAtOwner(OTI),fS,NULL); 
+      OwnerEntry *oe=receiveAtOwner(extOTI);
+      if (oe){
+	cellReceiveRead(oe,fS,NULL);
+      }
       break;
     }
   case M_CELL_REMOTEREAD:      
     {
-      int OTI;
+      Ext_OB_TIndex extOTI;
       DSite* fS,*mS;
-      msgC->get_M_CELL_REMOTEREAD(mS,OTI,fS);
+      msgC->get_M_CELL_REMOTEREAD(mS,extOTI,fS);
       PD((MSG_RECEIVED,"CELL_REMOTEREAD %s",fS->stringrep()));
-      cellReceiveRemoteRead(receiveAtBorrow(mS,OTI),mS,OTI,fS); 
+      cellReceiveRemoteRead(receiveAtBorrow(mS,extOTI),mS,extOTI,fS); 
       break;
     }
   case M_CELL_READANS:
     {
-      int index;
-      DSite*mS;
+      Ext_OB_TIndex index;
+      DSite *mS;
       TaggedRef val;
       msgC->get_M_CELL_READANS(mS,index,val);
       PD((MSG_RECEIVED,"CELL_READANS"));
+      // Erik, look
       OwnerEntry *oe=maybeReceiveAtOwner(mS,index);
       if(oe==NULL){
 	cellReceiveReadAns(receiveAtBorrow(mS,index)->getTertiary(),val);
@@ -880,102 +877,120 @@ void msgReceived(MsgContainer* msgC)
   case M_CELL_LOCK_FORWARD:
     {
       DSite* site,*rsite;
-      int OTI;
-      msgC->get_M_CELL_LOCK_FORWARD(site,OTI,rsite);
+      Ext_OB_TIndex extOTI;
+      msgC->get_M_CELL_LOCK_FORWARD(site,extOTI,rsite);
       PD((MSG_RECEIVED,"M_CELL_LOCK_FORWARD index:%d site:%s rsite:%s",
-	  OTI,site->stringrep(),rsite->stringrep()));
+	  extOTI,site->stringrep(),rsite->stringrep()));
 
-      cellLockReceiveForward(receiveAtBorrow(site,OTI),rsite,site,OTI);
+      cellLockReceiveForward(receiveAtBorrow(site, extOTI), rsite, site, extOTI);
       break;
     }
   case M_CELL_LOCK_DUMP:
     {
-      int OTI;
+      Ext_OB_TIndex extOTI;
       DSite* rsite;
-      msgC->get_M_CELL_LOCK_DUMP(OTI,rsite);
+      msgC->get_M_CELL_LOCK_DUMP(extOTI,rsite);
       PD((MSG_RECEIVED,"M_CELL_LOCK_DUMP index:%d site:%s",
-	  OTI,rsite->stringrep()));
-      cellLockReceiveDump(receiveAtOwner(OTI),rsite);
+	  extOTI,rsite->stringrep()));
+      OwnerEntry *oe=receiveAtOwner(extOTI);
+      if (oe)
+	{
+	  cellLockReceiveDump(oe,rsite);
+	}
       break;
     }
   case M_CELL_CANTPUT:
     {
       DSite* rsite, *ssite;
-      int OTI;
+      Ext_OB_TIndex extOTI;
       TaggedRef val;
-      msgC->get_M_CELL_CANTPUT( OTI, rsite, val, ssite);
+      msgC->get_M_CELL_CANTPUT(extOTI, rsite, val, ssite);
       PD((MSG_RECEIVED,"M_CELL_CANTPUT index:%d site:%s val:%s",
-	  OTI,rsite->stringrep(),toC(val)));
-      cellReceiveCantPut(receiveAtOwner(OTI),val,OTI,ssite,rsite);
+	  extOTI,rsite->stringrep(),toC(val)));
+      OwnerEntry *oe=receiveAtOwner(extOTI);
+      if (oe){
+	cellReceiveCantPut(oe,val,extOTI,ssite,rsite);
+      }
       break;
     }  
   case M_LOCK_TOKEN:
     {
       DSite* rsite;
-      int OTI;
-      msgC->get_M_LOCK_TOKEN(rsite,OTI);
+      Ext_OB_TIndex extOTI;
+      msgC->get_M_LOCK_TOKEN(rsite,extOTI);
       PD((MSG_RECEIVED,"M_LOCK_TOKEN index:%d site:%s",
-	  OTI,rsite->stringrep()));
-      OwnerEntry *oe=maybeReceiveAtOwner(rsite,OTI);
+	  extOTI,rsite->stringrep()));
+      // Erik, look
+      OwnerEntry *oe=maybeReceiveAtOwner(rsite,extOTI);
       if(oe!=NULL){
-	lockReceiveTokenManager(oe,OTI);
+	lockReceiveTokenManager(oe, extOTI);
 	break;}
-      lockReceiveTokenFrame(receiveAtBorrow(rsite,OTI),rsite,OTI);
+      lockReceiveTokenFrame(receiveAtBorrow(rsite, extOTI), rsite, extOTI);
       break;
     }
   case M_CHAIN_ACK:
     {
-      int OTI;
+      Ext_OB_TIndex extOTI;
       DSite* rsite;
-      msgC->get_M_CHAIN_ACK(OTI,rsite);
+      msgC->get_M_CHAIN_ACK(extOTI,rsite);
       PD((MSG_RECEIVED,"M_CHAIN_ACK index:%d site:%s",
-	  OTI,rsite->stringrep()));
-      chainReceiveAck(receiveAtOwner(OTI),rsite);
+	  extOTI,rsite->stringrep()));
+      OwnerEntry *oe=receiveAtOwner(extOTI);
+      if (oe){
+	chainReceiveAck(oe,rsite);
+      }
       break;
     }
   case M_LOCK_CANTPUT:
     {
       DSite* rsite, *ssite;
-      int OTI;
-      msgC->get_M_LOCK_CANTPUT( OTI, rsite, ssite);
+      Ext_OB_TIndex extOTI;
+      msgC->get_M_LOCK_CANTPUT( extOTI, rsite, ssite);
       PD((MSG_RECEIVED,"M_LOCK_CANTPUT index:%d site:%s val:%s",
-	  OTI,rsite->stringrep()));
-      lockReceiveCantPut(receiveAtOwner(OTI),OTI,ssite,rsite);
+	  extOTI,rsite->stringrep()));
+      OwnerEntry *oe=receiveAtOwner(extOTI);
+      if(oe)
+	{
+	  lockReceiveCantPut(oe,extOTI,ssite,rsite);
+	}
       break;
     }
   case M_CHAIN_QUESTION:
    {
       DSite* site,*deadS;
-      int OTI;
-      msgC->get_M_CHAIN_QUESTION(OTI,site,deadS);
+      Ext_OB_TIndex extOTI;
+      msgC->get_M_CHAIN_QUESTION(extOTI,site,deadS);
       PD((MSG_RECEIVED,"M_CHAIN_QUESTION index:%d site:%s",
-	  OTI,site->stringrep()));
-      BorrowEntry *be=maybeReceiveAtBorrow(site,OTI);
+	  extOTI,site->stringrep()));
+      BorrowEntry *be=maybeReceiveAtBorrow(site,extOTI);
       if(be==NULL) break;
-      chainReceiveQuestion(be,site,OTI,deadS);
+      chainReceiveQuestion(be,site,extOTI,deadS);
       break;
    }
   case M_CHAIN_ANSWER:
     {
       DSite* rsite,*deadS;
-      int OTI;
+      Ext_OB_TIndex extOTI;
       int ans;
-      msgC->get_M_CHAIN_ANSWER(OTI,rsite,ans,deadS);
+      msgC->get_M_CHAIN_ANSWER(extOTI,rsite,ans,deadS);
       PD((MSG_RECEIVED,"M_CHAIN_ANSWER index:%d site:%s val:%d",
-	  OTI,rsite->stringrep(),ans));
-      chainReceiveAnswer(receiveAtOwner(OTI),rsite,ans,deadS);
+	  extOTI,rsite->stringrep(),ans));
+      OwnerEntry *oe=receiveAtOwner(extOTI);
+      if(oe){
+	chainReceiveAnswer(oe,rsite,ans,deadS);
+      }
       break;
     }  
 
   case M_TELL_ERROR:
     {
       DSite* site;
-      int OTI;
+      Ext_OB_TIndex extOTI;
       int ec,flag;
-      msgC->get_M_TELL_ERROR(site,OTI,ec,flag);
+      msgC->get_M_TELL_ERROR(site,extOTI,ec,flag);
       PD((MSG_RECEIVED,"M_TELL_ERROR index:%d site:%s ec:%d",
-	  OTI,site->stringrep(),ec));
-      BorrowEntry *be=maybeReceiveAtBorrow(site,OTI);
+	  extOTI,site->stringrep(),ec));
+      BorrowEntry *be=maybeReceiveAtBorrow(site,extOTI);
       if(be==NULL) break;
       receiveTellError(be,ec,flag);
       break;
@@ -983,24 +998,30 @@ void msgReceived(MsgContainer* msgC)
 
   case M_ASK_ERROR:
     {
-      int OTI;
+      Ext_OB_TIndex extOTI;
       int ec;
       DSite* toS;
-      msgC->get_M_ASK_ERROR(OTI,toS,ec);
+      msgC->get_M_ASK_ERROR(extOTI,toS,ec);
       PD((MSG_RECEIVED,"M_ASK_ERROR index:%d ec:%d toS:%s",
-	  OTI,ec,toS->stringrep()));
-      receiveAskError(receiveAtOwner(OTI),toS,ec);
+	  extOTI,ec,toS->stringrep()));
+      OwnerEntry *oe=receiveAtOwner(extOTI);
+      if (oe){
+	receiveAskError(oe,toS,ec);
+      }
       break; 
     }
   case M_UNASK_ERROR:
     {
-      int OTI;
+      Ext_OB_TIndex extOTI;
       int ec;
       DSite* toS;
-      msgC->get_M_UNASK_ERROR(OTI,toS,ec);
+      msgC->get_M_UNASK_ERROR(extOTI,toS,ec);
       PD((MSG_RECEIVED,"M_UNASK_ERROR index:%d ec:%d toS:%s",
-	  OTI,ec,toS->stringrep()));
-      receiveUnAskError(receiveAtOwner(OTI),toS,ec);
+	  extOTI,ec,toS->stringrep()));
+      
+      OwnerEntry *oe=receiveAtOwner(extOTI);
+      if (oe)
+	receiveUnAskError(oe,toS,ec);
       break; 
     }
   case M_PING:
@@ -1009,10 +1030,47 @@ void msgReceived(MsgContainer* msgC)
       // that is or will be sent is enough.
       break;
     }
+
+  case M_PONG_TERM:
+    {
+      DSite *dest;
+      int ctr,id; 
+      OZ_Term trm;
+      msgC->get_M_PONG_TERM(dest,ctr,id,trm);
+      ctr --; 
+      if (ctr == 0) 
+	ResumeSendJobb(id);
+      else
+	{
+	  MsgContainer *msgC = msgContainerManager->newMsgContainer(dest);
+	  msgC->put_M_PONG_TERM(myDSite,ctr, id,trm);
+	  send(msgC);
+	}
+      break;
+    }
+    
+  case M_PONG_PL:
+    {
+      DSite *dest;
+      int ctr,id; 
+      msgC->get_M_PONG_PL(dest,ctr,id);
+      ctr --;
+      if (ctr == 0) 
+	ResumeSendJobb(id);
+      else
+	{
+	  MsgContainer *msgC = msgContainerManager->newMsgContainer(dest);
+	  msgC->put_M_PONG_PL(myDSite,ctr, id);
+	  send(msgC);
+	}
+      break;
+    }
   default:
     OZ_error("siteReceive: unknown message %d\n",mt);
     break;
   }
+
+  DebugCode(OT->checkEntries());
 }
 
 
@@ -1034,8 +1092,9 @@ enum CommCase{
     USUAL_BORROW_CASE
   };
 
-void DSite::communicationProblem(MsgContainer *msgC, FaultCode fc) {
-  int OTI;
+void DSite::communicationProblem(MsgContainer *msgC, FaultCode fc)
+{
+  Ext_OB_TIndex extOTI;
   DSite* s1;
   TaggedRef tr;
   CommCase flag;
@@ -1043,8 +1102,8 @@ void DSite::communicationProblem(MsgContainer *msgC, FaultCode fc) {
   switch(msgC->getMessageType()) {
   case M_CELL_CONTENTS: {
     if(fc == COMM_FAULT_PERM_NOT_SENT) {
-      msgC->get_M_CELL_CONTENTS(s1,OTI,tr);
-      cellSendContentsFailure(tr,this,s1,OTI);
+      msgC->get_M_CELL_CONTENTS(s1,extOTI,tr);
+      cellSendContentsFailure(tr, this, s1, extOTI);
       return;
     }
     flag=USUAL_BORROW_CASE;
@@ -1053,8 +1112,8 @@ void DSite::communicationProblem(MsgContainer *msgC, FaultCode fc) {
 
   case M_LOCK_TOKEN: {
     if(fc == COMM_FAULT_PERM_NOT_SENT) {
-      msgC->get_M_LOCK_TOKEN(s1,OTI);
-      lockSendTokenFailure(this,s1,OTI);
+      msgC->get_M_LOCK_TOKEN(s1,extOTI);
+      lockSendTokenFailure(this, s1, extOTI);
       return;
     }
     return;
@@ -1079,6 +1138,7 @@ OZ_BI_proto(BIfailureDefault);
 //  DebugCode(FILE *log;)
 void initDPCore()
 {
+  genFreeListManager = new GenFreeListManager();
   // link interface...
 //    DebugCode(char *s;sprintf(s,"~/tmp/%d",osgetpid()));
 //    DebugCode(log=freopen(s,"w",stdout);)
@@ -1115,17 +1175,11 @@ void initDPCore()
 
   initNetwork();
 
-  borrowTable      = new BorrowTable(DEFAULT_BORROW_TABLE_SIZE);
-  ownerTable       = new OwnerTable(DEFAULT_OWNER_TABLE_SIZE);
+  borrowTable      = new BorrowTable(ozconf.dpTableDefaultBorrowTableSize);
+  ownerTable       = new OwnerTable(ozconf.dpTableDefaultOwnerTableSize);
   resourceTable    = new ResourceHashTable(RESOURCE_HASH_TABLE_DEFAULT_SIZE);
-  flowControler    = new FlowControler();
   //  msgBufferManager = new MarshalerBufferManager();
 
-#ifndef DENYS_EVENTS
-  if(!am.registerTask((void*)flowControler, FlowControlCheck, FlowControlExecute))
-    OZ_error("Unable to register FlowControl task");
-#endif
-   
   BI_defer = makeTaggedConst(new Builtin("", "defer", 0, 0, BIdefer, OK));
   globalWatcher = NULL; 
   DeferdEvents = NULL;
@@ -1165,20 +1219,6 @@ void initDPCore()
 /*   MISC                                                */
 /**********************************************************************/
 
-void marshalDSite(MarshalerBuffer *buf, DSite* s)
-{
-  s->marshalDSite(buf);
-}
-
-DSite* getSiteFromBTI(int i){
-  return BT->getBorrow(i)->getNetAddress()->site;}
-
-OwnerEntry *getOwnerEntryFromOTI(int i){
-  return OT->getOwner(i);}
-
-Tertiary* getTertiaryFromOTI(int i){
-  return OT->getOwner(i)->getTertiary();}
-
 /*
  * The builtin table: no builtins, just a fake
  */
@@ -1189,7 +1229,6 @@ Tertiary* getTertiaryFromOTI(int i){
 OZ_BI_proto(BIdp_task_tmpDown);
 OZ_BI_proto(BIdp_task_myDown);
 OZ_BI_proto(BIdp_task_probe);
-OZ_BI_proto(BIdp_task_flowControl);
 #endif
 
 extern "C"
@@ -1201,7 +1240,6 @@ extern "C"
       {"task.tmpDown",0,1,BIdp_task_tmpDown},
       {"task.myDown",0,1,BIdp_task_myDown},
       {"task.probe",0,1,BIdp_task_probe},
-      {"task.flowControl",0,1,BIdp_task_flowControl},
 #endif
       {0,0,0,0}
     };
@@ -1222,19 +1260,19 @@ extern "C"
 /**********************************************************************/
 
 OZ_Term getGatePort(DSite* sd){
-  int si=0; /* Gates are always located at position 0 */
+  /* Gates are always located at position 0 */
+  Ext_OB_TIndex si = (Ext_OB_TIndex) 0;
   if(sd==myDSite){
-    OwnerEntry* oe=OT->getOwner(si);
+    OwnerEntry* oe = OT->extOTI2ownerEntry(si);
     Assert(oe->isPersistent());
     return  oe->getValue();}
-  NetAddress na = NetAddress(sd,si); 
-  BorrowEntry *b = borrowTable->find(&na);
+  BorrowEntry *b = borrowTable->find(si, sd);
   if (b==NULL) {
-    int bi=borrowTable->newBorrowPersistent(sd,si);
-    b=borrowTable->getBorrow(bi);
+    OB_TIndex bi = borrowTable->newBorrow(NULL, sd, si);
+    b=borrowIndex2borrowEntry(bi);
     PortProxy *pp = new PortProxy(bi);
     b->mkTertiary(pp);
-//      b->makePersistent(); // Already is
+
     if(sd->siteStatus()!=SITE_OK){
       if(sd->siteStatus()==SITE_PERM){
 	deferProxyTertProbeFault(pp,PROBE_PERM);}
@@ -1287,6 +1325,7 @@ void dpExitWithTimer(unsigned int timeUntilClose)
     osBlockSelect(ts);
     osBlockSignals(NO);
     timeUntilClose -= (osTotalTime() - idle_start);
+    oz_io_check();
     oz_io_handle();
   } while ((int) timeUntilClose > 0 && remaining);
   */
@@ -1343,6 +1382,7 @@ void dpExitWithTimer(unsigned int timeUntilClose)
     osBlockSignals(NO);
     timeUntilClose -= (osTotalTime() - idle_start);
     am.doGCollect();
+    oz_io_check();
     oz_io_handle();
   } while ((int) timeUntilClose > 0);
   timeUntilClose = tuc;
@@ -1361,11 +1401,13 @@ void dpExitWithTimer(unsigned int timeUntilClose)
 //        fprintf(stderr, "... closing, remaining %d frames (pid %d)!\n",
 //  	      remaining, osgetpid());
 //        fflush(stderr);
+      oz_io_check();
       oz_io_handle();
       osUnblockSignals();
       unsigned int ts = TIME_SLICE;
       osBlockSelect(ts);
       osBlockSignals(NO);
+      oz_io_check();
       oz_io_handle();
       timeUntilClose -= (osTotalTime() - idle_start);
     } while ((int) timeUntilClose > 0 && remaining);
@@ -1378,6 +1420,7 @@ void dpExitWithTimer(unsigned int timeUntilClose)
     timeUntilClose = tuc;
     do {
       unsigned long idle_start = osTotalTime();
+      oz_io_check();
       oz_io_handle();
       osUnblockSignals();
       unsigned int ts = TIME_SLICE;
@@ -1404,6 +1447,7 @@ void dpExitWithTimer(unsigned int timeUntilClose)
     osBlockSelect(ts);
     osBlockSignals(NO);
     timeUntilClose -= (osTotalTime() - idle_start);
+    oz_io_check();
     oz_io_handle();
   }
   */
@@ -1432,4 +1476,5 @@ void dpExitImpl() {
   dpExitWithTimer(ozconf.closetime);
 }
 
-
+// see memaux.hh;
+GenFreeListManager *genFreeListManager;
