@@ -9,8 +9,8 @@
  *    Konstantin Popov (2000)
  * 
  *  Last change:
- *    $Date: 2003/04/28 12:32:28 $ by $Author: glynn $
- *    $Revision: 1.7 $
+ *    $Date: 2006/05/30 13:03:01 $ by $Author: mejias $
+ *    $Revision: 1.7.8.1 $
  * 
  *  This file is part of Mozart, an implementation 
  *  of Oz 3:
@@ -47,14 +47,14 @@ void ClassVar::marshal(ByteBuffer *bs)
 {
   PD((MARSHAL,"var classproxy"));
   // lazy classes are not first class now, so:
-  OZ_error("marshaling a ClassVar??!");
+  OZ_error("marshaling a ClassVar?!?");
 }
 
 //
 void ClassVar::sendRequest()
 {
   // lazy classes are not first class now, so:
-  OZ_error("requesting the class of a ClassVar??!");
+  OZ_error("requesting the class of a ClassVar?!?");
 }
 
 //
