@@ -8,8 +8,8 @@
  *    Konstantin Popov 1997-1998
  * 
  *  Last change:
- *    $Date: 2002/10/19 12:51:47 $ by $Author: duchier $
- *    $Revision: 1.18.6.1 $
+ *    $Date: 2003/04/28 12:32:28 $ by $Author: glynn $
+ *    $Revision: 1.20 $
  * 
  *  This file is part of Mozart, an implementation 
  *  of Oz 3:
@@ -488,18 +488,9 @@ void VirtualSite::marshalLocalResources(MarshalerBuffer *mb,
 
 //
 // The 'segKeys' array is allocated with "
-#ifndef USE_FAST_UNMARSHALER
-void VirtualSite::unmarshalResourcesRobust(MarshalerBuffer *mb, int *error)
-#else
 void VirtualSite::unmarshalResources(MarshalerBuffer *mb)
-#endif
 {
-#ifndef USE_FAST_UNMARSHALER
-  segKeysNum = unmarshalNumberRobust(mb, error);
-  if(*error) return;
-#else
   segKeysNum = unmarshalNumber(mb);
-#endif
   Assert(segKeysNum);
   if (segKeysNum > segKeysArraySize) {
     int acc = segKeysNum, bits = 0;
@@ -518,15 +509,8 @@ void VirtualSite::unmarshalResources(MarshalerBuffer *mb)
   Assert(segKeysArraySize >= segKeysNum);
 
   //
-#ifndef USE_FAST_UNMARSHALER
-  for (int i = 0; i < segKeysNum; i++) {
-    segKeys[i] = (key_t) unmarshalNumberRobust(mb, error);
-    if(*error) return;
-  }
-#else
   for (int i = 0; i < segKeysNum; i++) 
     segKeys[i] = (key_t) unmarshalNumber(mb);
-#endif
 }
 
 //
