@@ -10,8 +10,8 @@
  *    Organization or Person (Year(s))
  * 
  *  Last change:
- *    $Date: 2002/08/20 14:19:47 $ by $Author: kornstae $
- *    $Revision: 1.29.2.2 $
+ *    $Date: 2003/09/05 09:28:25 $ by $Author: duchier $
+ *    $Revision: 1.32 $
  * 
  *  This file is part of Mozart, an implementation 
  *  of Oz 3:
@@ -30,7 +30,7 @@
 #include <stdlib.h>
 #include <math.h>
 
-#if __GLIBC__ == 2 || defined(WINDOWS) || defined(NETBSD) || defined(RHAPSODY) || defined(OPENBSD)
+#if __GLIBC__ == 2 || defined(WINDOWS) || defined(NETBSD) || defined(RHAPSODY) || defined(OPENBSD) || defined(__FreeBSD__)
 
 /* the following have been copied from linux's values.h */
 #define _DEXPLEN    11
