@@ -12,8 +12,8 @@
  *    Christian Schulte, 1999
  * 
  *  Last change:
- *    $Date: 2002/10/19 12:51:46 $ by $Author: duchier $
- *    $Revision: 1.11.2.1.2.1 $
+ *    $Date: 2003/12/22 11:23:47 $ by $Author: raph $
+ *    $Revision: 1.15 $
  * 
  *  This file is part of Mozart, an implementation 
  *  of Oz 3:
@@ -346,10 +346,13 @@ OZ_BI_define(BIwidthC, 2, 0)
 	    break;
 	  }
 
-	case EVAR_STATUS_FUTURE:
+	case EVAR_STATUS_READONLY:
+	case EVAR_STATUS_FAILED:
 	case EVAR_STATUS_DET:
 	case EVAR_STATUS_UNKNOWN:
 	  // kost@ : wait until the picture gets clear;
+	  // raph: the following is fine for failed values, because the
+	  // attempt to suspend will raise the failed value's exception.
 	  oz_suspendOn(rawwid);
 
 	case EVAR_STATUS_KINDED:
