@@ -10,8 +10,8 @@
  *    Organization or Person (Year(s))
  * 
  *  Last change:
- *    $Date: 2002/10/19 12:51:46 $ by $Author: duchier $
- *    $Revision: 1.294.2.3.2.1 $
+ *    $Date: 2003/06/04 21:26:30 $ by $Author: duchier $
+ *    $Revision: 1.298 $
  * 
  *  This file is part of Mozart, an implementation 
  *  of Oz 3:
@@ -428,7 +428,7 @@ void OzVariable::printLongStream(ostream &stream, int depth, int offset)
 class Order_Taggedref_By_Feat {
 public:
   Bool operator()(const TaggedRef& a, const TaggedRef& b) {
-    return featureCmp(a,b) <= 0;
+    return featureCmp(a,b) < 0;
   }
 };
 
