@@ -11,8 +11,8 @@
  *    Christian Schulte, 1997
  * 
  *  Last change:
- *    $Date: 2001/12/06 15:06:24 $ by $Author: duchier $
- *    $Revision: 1.193.2.8 $
+ *    $Date: 2004/05/17 14:25:19 $ by $Author: duchier $
+ *    $Revision: 1.202.2.1 $
  * 
  *  This file is part of Mozart, an implementation 
  *  of Oz 3:
@@ -539,6 +539,9 @@ inline OZ_Return buffer_vs(OZ_Term vs, char *write_buff, int *len,
   return vs2buff(vs, &write_buff, len, rest, susp);
 }
 
+OZ_Return OZ_buffer_vs(OZ_Term vs, char *write_buff, int *len,
+		       OZ_Term *rest, OZ_Term *susp)
+{ return buffer_vs(vs,write_buff,len,rest,susp); }
 
 // -------------------------------------------------
 // unix IO
@@ -628,11 +631,14 @@ OZ_BI_iodefine(unix_stat,1,1)
   else 
     fileType = "unknown";
 
-  OZ_MAKE_RECORD_S("stat",3,
-		   {"type" OZ_COMMA "size" OZ_COMMA "mtime"},
+  OZ_MAKE_RECORD_S("stat",5,
+		   {"type" OZ_COMMA "size" OZ_COMMA "mtime"
+		      OZ_COMMA "ino" OZ_COMMA "dev"},
 		   {oz_atom(fileType) OZ_COMMA 
 		      oz_int(buf.st_size) OZ_COMMA
-		      oz_int(buf.st_mtime)}, r);
+		      oz_int(buf.st_mtime) OZ_COMMA
+		      oz_int(buf.st_ino) OZ_COMMA
+		      oz_int(buf.st_dev)}, r);
 
   OZ_RETURN(r);
 } OZ_BI_ioend
@@ -1406,6 +1412,44 @@ OZ_BI_iodefine(unix_accept_nonblocking,1,3)
 } OZ_BI_ioend
 
 
+/* This has the same functionality as unix_accept_nonblocking but with
+   a small difference. That is, there is no name service query. The
+   Host output parameter will be a string representing the IP of the
+   end-host. */
+OZ_BI_iodefine(unix_accept_nonblocking_noDnsLookup,1,3)
+{
+  OZ_declareInt(0, sock);
+  // OZ_out(0) == host
+  // OZ_out(1) == port
+  // OZ_out(2) == fd
+
+  struct sockaddr_in from;
+  int fromlen = sizeof from;
+
+  WRAPCALL("accept",osaccept(sock,(struct sockaddr *)&from, &fromlen),fd);
+
+  // Nonblocking
+  int one=1;
+  if(setsockopt(fd,IPPROTO_TCP,TCP_NODELAY,(char*) &one,sizeof(one))<0){
+    RETURN_UNIX_ERROR("acceptNonblocking");
+  }
+
+#ifdef WINDOWS
+  osSetNonBlocking(fd,OK);
+#else
+  fcntl(fd,F_SETFL,O_NDELAY);
+#endif
+  //
+
+  char *host = inet_ntoa(from.sin_addr);
+
+  OZ_out(0) = OZ_string(host);
+  OZ_out(1) = OZ_int(ntohs(from.sin_port));
+  OZ_out(2) = OZ_int(fd);
+  return PROCEED;
+} OZ_BI_ioend
+
+
 static OZ_Return get_send_recv_flags(OZ_Term OzFlags, int * flags)
 {
   OZ_Term hd, tl;
@@ -1856,7 +1900,8 @@ OZ_BI_define(unix_pipe,2,2) {
        *   this allows to press Control-C when debugging the emulator
        */
       if (setsid() < 0) {
-        RETURN_UNIX_ERROR("setsid");
+	fprintf(stderr,"setsid failed\n");
+	exit(-1);
       }
 #endif
 
@@ -1867,7 +1912,8 @@ OZ_BI_define(unix_pipe,2,2) {
       rlim.rlim_cur = 0;
       rlim.rlim_max = 0;
       if (setrlimit(RLIMIT_CORE, &rlim) < 0) {
-	RETURN_UNIX_ERROR("setrlimit");
+	fprintf(stderr,"setrlimit failed\n");
+	exit(-1);
       }
 
       for (i = 0; i < FD_SETSIZE; i++) {
@@ -1879,9 +1925,10 @@ OZ_BI_define(unix_pipe,2,2) {
       osdup(sv[1]);
       osdup(sv[1]);
       if (execvp(s,argv)  < 0) {
-        RETURN_UNIX_ERROR("execvp");
+	fprintf(stderr,"execvp failed\n");
+	exit(-1);
       }
-      printf("execvp failed\n");
+      printf("this should never happen\n");
       exit(-1);
     }
   case -1:
@@ -2110,6 +2157,12 @@ OZ_BI_iodefine(unix_getHostByName, 1,1)
 
 // Misc stuff
 
+OZ_BI_iodefine(unix_rmDir,1,0) {
+  OZ_declareVsIN(0,path);
+  WRAPCALL("rmdir",rmdir(path),ret);
+  return PROCEED;
+} OZ_BI_ioend
+
 OZ_BI_iodefine(unix_unlink, 1,0) {
   OZ_declareVsIN(0,path);
 
