@@ -9,8 +9,8 @@
  *    Organization or Person (Year(s))
  * 
  *  Last change:
- *    $Date: 2000/09/15 06:54:21 $ by $Author: annan $
- *    $Revision: 1.40 $
+ *    $Date: 2005/05/19 17:48:45 $ by $Author: duchier $
+ *    $Revision: 1.40.18.1 $
  * 
  *  This file is part of Mozart, an implementation 
  *  of Oz 3:
@@ -203,7 +203,7 @@ static char* URLC_kp[] = {"http://", "file:", "ftp://", NULL};
 /* characters not to be escaped in HTTP requests.
    if I well understood RFC 1945! ##
    */
-static char URLC_hs[] = "%:@&=+$-_.!*'(),;/?";
+static char URLC_hs[] = "%:@&=+$-_.!*'(),;/?#";
 
 /* throw simulators */
 #define th1(reason) { clean(); return (reason); }
