@@ -6,13 +6,15 @@
  * 
  *  Contributors:
  *    Christian Schulte <schulte@ps.uni-sb.de>
+ *    Raphael Collet <raph@info.ucl.ac.be>
+ *    Alfred Spiessens <fsp@info.ucl.ac.be>
  * 
  *  Copyright:
  *    Organization or Person (Year(s))
  * 
  *  Last change:
- *    $Date: 2002/10/19 12:51:47 $ by $Author: duchier $
- *    $Revision: 1.75.2.2.2.1 $
+ *    $Date: 2004/04/08 01:11:20 $ by $Author: popow $
+ *    $Revision: 1.82 $
  * 
  *  This file is part of Mozart, an implementation 
  *  of Oz 3:
@@ -39,7 +41,8 @@
 #include "var_ct.hh"
 #include "var_simple.hh"
 #include "var_opt.hh"
-#include "var_future.hh"
+#include "var_readonly.hh"
+#include "var_failed.hh"
 #include "var_ext.hh"
 #include "dpInterface.hh"
 #include "ozconfig.hh"
@@ -67,31 +70,44 @@ void OzVariable::dropPropagator(Propagator * prop)
 
 Bool oz_var_valid(OzVariable *ov,TaggedRef val) {
   switch (ov->getType()){
-  case OZ_VAR_OPT:     return ((OptVar *) ov)->valid(val);
-  case OZ_VAR_SIMPLE:  return ((SimpleVar *) ov)->valid(val);
-  case OZ_VAR_FUTURE:  return ((Future *) ov)->valid(val);
-  case OZ_VAR_BOOL:    return ((OzBoolVariable*) ov)->valid(val);
-  case OZ_VAR_FD:      return ((OzFDVariable*) ov)->valid(val);
-  case OZ_VAR_OF:      return ((OzOFVariable*) ov)->valid(val);
-  case OZ_VAR_FS:      return ((OzFSVariable*) ov)->valid(val);
-  case OZ_VAR_CT:      return ((OzCtVariable*) ov)->valid(val);
-  case OZ_VAR_EXT:     return var2ExtVar(ov)->validV(val);
+  case OZ_VAR_OPT:      return ((OptVar *) ov)->valid(val);
+  case OZ_VAR_SIMPLE_QUIET:
+  case OZ_VAR_SIMPLE:   return ((SimpleVar *) ov)->valid(val);
+  case OZ_VAR_READONLY_QUIET:
+  case OZ_VAR_READONLY: return ((ReadOnly *) ov)->valid(val);
+  case OZ_VAR_FAILED:   return ((Failed *) ov)->valid(val);
+  case OZ_VAR_BOOL:     return ((OzBoolVariable*) ov)->valid(val);
+  case OZ_VAR_FD:       return ((OzFDVariable*) ov)->valid(val);
+  case OZ_VAR_OF:       return ((OzOFVariable*) ov)->valid(val);
+  case OZ_VAR_FS:       return ((OzFSVariable*) ov)->valid(val);
+  case OZ_VAR_CT:       return ((OzCtVariable*) ov)->valid(val);
+  case OZ_VAR_EXT:      return var2ExtVar(ov)->validV(val);
   ExhaustiveSwitch();
   }
   return NO;
 }
 
-OZ_Return oz_var_unify(OzVariable *ov,TaggedRef *ptr,TaggedRef *val) {
+OZ_Return oz_var_unify(OzVariable *ov, TaggedRef *ptr, TaggedRef *val) {
+  Assert(oz_isVar(*val));
+  Assert(ov == tagged2Var(*ptr));
+  DebugCode(OzVariable *rv = tagged2Var(*val));
+  DebugCode(Board* tb1 = ov->getBoardInternal()->derefBoard());
+  DebugCode(Board* tb2 = rv->getBoardInternal()->derefBoard());
+  Assert(oz_isBelow(tb1, tb2) || ov->getType() >= rv->getType());
+  //
   switch (ov->getType()){
-  case OZ_VAR_OPT:     return ((OptVar *) ov)->unify(ptr,val);
-  case OZ_VAR_SIMPLE:  return ((SimpleVar *) ov)->unify(ptr,val);
-  case OZ_VAR_FUTURE:  return ((Future *) ov)->unify(ptr,val);
-  case OZ_VAR_BOOL:    return ((OzBoolVariable*) ov)->unify(ptr,val);
-  case OZ_VAR_FD:      return ((OzFDVariable*) ov)->unify(ptr,val);
-  case OZ_VAR_OF:      return ((OzOFVariable*) ov)->unify(ptr,val);
-  case OZ_VAR_FS:      return ((OzFSVariable*) ov)->unify(ptr,val);
-  case OZ_VAR_CT:      return ((OzCtVariable*) ov)->unify(ptr,val);
-  case OZ_VAR_EXT:     return var2ExtVar(ov)->unifyV(ptr,val);
+  case OZ_VAR_OPT:      return ((OptVar *) ov)->unify(ptr,val);
+  case OZ_VAR_SIMPLE_QUIET:
+  case OZ_VAR_SIMPLE:   return ((SimpleVar *) ov)->unify(ptr,val);
+  case OZ_VAR_READONLY_QUIET:
+  case OZ_VAR_READONLY: return ((ReadOnly *) ov)->unify(ptr,val);
+  case OZ_VAR_FAILED:   return ((Failed *) ov)->unify(ptr,val);
+  case OZ_VAR_BOOL:     return ((OzBoolVariable*) ov)->unify(ptr,val);
+  case OZ_VAR_FD:       return ((OzFDVariable*) ov)->unify(ptr,val);
+  case OZ_VAR_OF:       return ((OzOFVariable*) ov)->unify(ptr,val);
+  case OZ_VAR_FS:       return ((OzFSVariable*) ov)->unify(ptr,val);
+  case OZ_VAR_CT:       return ((OzCtVariable*) ov)->unify(ptr,val);
+  case OZ_VAR_EXT:      return var2ExtVar(ov)->unifyV(ptr,val);
   ExhaustiveSwitch();
   }
   return FAILED;
@@ -99,15 +115,18 @@ OZ_Return oz_var_unify(OzVariable *ov,TaggedRef *ptr,TaggedRef *val) {
 
 OZ_Return oz_var_bind(OzVariable *ov,TaggedRef *ptr,TaggedRef val) {
   switch (ov->getType()){
-  case OZ_VAR_OPT:     return ((OptVar *) ov)->bind(ptr,val);
-  case OZ_VAR_SIMPLE:  return ((SimpleVar *) ov)->bind(ptr,val);
-  case OZ_VAR_FUTURE:  return ((Future *) ov)->bind(ptr,val);
-  case OZ_VAR_BOOL:    return ((OzBoolVariable*) ov)->bind(ptr,val);
-  case OZ_VAR_FD:      return ((OzFDVariable*) ov)->bind(ptr,val);
-  case OZ_VAR_OF:      return ((OzOFVariable*) ov)->bind(ptr,val);
-  case OZ_VAR_FS:      return ((OzFSVariable*) ov)->bind(ptr,val);
-  case OZ_VAR_CT:      return ((OzCtVariable*) ov)->bind(ptr,val);
-  case OZ_VAR_EXT:     return var2ExtVar(ov)->bindV(ptr,val);
+  case OZ_VAR_OPT:      return ((OptVar *) ov)->bind(ptr,val);
+  case OZ_VAR_SIMPLE_QUIET:
+  case OZ_VAR_SIMPLE:   return ((SimpleVar *) ov)->bind(ptr,val);
+  case OZ_VAR_READONLY_QUIET:
+  case OZ_VAR_READONLY: return ((ReadOnly *) ov)->bind(ptr,val);
+  case OZ_VAR_FAILED:   return ((Failed *) ov)->bind(ptr,val);
+  case OZ_VAR_BOOL:     return ((OzBoolVariable*) ov)->bind(ptr,val);
+  case OZ_VAR_FD:       return ((OzFDVariable*) ov)->bind(ptr,val);
+  case OZ_VAR_OF:       return ((OzOFVariable*) ov)->bind(ptr,val);
+  case OZ_VAR_FS:       return ((OzFSVariable*) ov)->bind(ptr,val);
+  case OZ_VAR_CT:       return ((OzCtVariable*) ov)->bind(ptr,val);
+  case OZ_VAR_EXT:      return var2ExtVar(ov)->bindV(ptr,val);
   ExhaustiveSwitch();
   }
   return FAILED;
@@ -115,15 +134,18 @@ OZ_Return oz_var_bind(OzVariable *ov,TaggedRef *ptr,TaggedRef val) {
 
 OZ_Return oz_var_forceBind(OzVariable *ov,TaggedRef *ptr,TaggedRef val) {
   switch (ov->getType()){
-  case OZ_VAR_OPT:     return ((OptVar *) ov)->bind(ptr,val);
-  case OZ_VAR_SIMPLE:  return ((SimpleVar *) ov)->bind(ptr,val);
-  case OZ_VAR_FUTURE:  return ((Future *) ov)->forceBind(ptr,val);
-  case OZ_VAR_BOOL:    return ((OzBoolVariable*) ov)->bind(ptr,val);
-  case OZ_VAR_FD:      return ((OzFDVariable*) ov)->bind(ptr,val);
-  case OZ_VAR_OF:      return ((OzOFVariable*) ov)->bind(ptr,val);
-  case OZ_VAR_FS:      return ((OzFSVariable*) ov)->bind(ptr,val);
-  case OZ_VAR_CT:      return ((OzCtVariable*) ov)->bind(ptr,val);
-  case OZ_VAR_EXT:     return var2ExtVar(ov)->forceBindV(ptr,val);
+  case OZ_VAR_OPT:      return ((OptVar *) ov)->bind(ptr,val);
+  case OZ_VAR_SIMPLE_QUIET:
+  case OZ_VAR_SIMPLE:   return ((SimpleVar *) ov)->bind(ptr,val);
+  case OZ_VAR_READONLY_QUIET:
+  case OZ_VAR_READONLY: return ((ReadOnly *) ov)->forceBind(ptr,val);
+  case OZ_VAR_FAILED:   return ((Failed *) ov)->forceBind(ptr,val);
+  case OZ_VAR_BOOL:     return ((OzBoolVariable*) ov)->bind(ptr,val);
+  case OZ_VAR_FD:       return ((OzFDVariable*) ov)->bind(ptr,val);
+  case OZ_VAR_OF:       return ((OzOFVariable*) ov)->bind(ptr,val);
+  case OZ_VAR_FS:       return ((OzFSVariable*) ov)->bind(ptr,val);
+  case OZ_VAR_CT:       return ((OzCtVariable*) ov)->bind(ptr,val);
+  case OZ_VAR_EXT:      return var2ExtVar(ov)->forceBindV(ptr,val);
   ExhaustiveSwitch();
   }
   return FAILED;
@@ -145,14 +167,23 @@ OZ_Return oz_var_addSusp(TaggedRef *v, Suspendable * susp) {
   Assert(oz_isVar(*v));
   OzVariable *ov = tagged2Var(*v);
   switch (ov->getType()) {
-  case OZ_VAR_FUTURE:
-    return ((Future *) ov)->addSusp(v, susp);
+  case OZ_VAR_FAILED:
+    return ((Failed *) ov)->addSusp(v, susp);
+  case OZ_VAR_READONLY_QUIET:
+    ((ReadOnly*) ov)->becomeNeeded();
+    // fall through
+  case OZ_VAR_READONLY:
+    ov->addSuspSVar(susp);
+    return (SUSPEND);
   case OZ_VAR_EXT:
     return var2ExtVar(ov)->addSuspV(v, susp);
   case OZ_VAR_OPT:
     ov = new SimpleVar(ov->getBoardInternal());
     *v = makeTaggedVar(ov);
-    // fall through;
+    // fall through
+  case OZ_VAR_SIMPLE_QUIET:
+    ((SimpleVar*) ov)->becomeNeeded();
+    // fall through
   case OZ_VAR_SIMPLE:
     if (ozconf.useFutures || susp->isNoBlock()) {
       return oz_raise(E_ERROR, E_KERNEL, "block", 1, makeTaggedRef(v));
@@ -164,17 +195,69 @@ OZ_Return oz_var_addSusp(TaggedRef *v, Suspendable * susp) {
   }
 }
 
+// fred+raph: Add a "quiet" suspension to v
+OZ_Return oz_var_addQuietSusp(TaggedRef *v, Suspendable * susp) {
+  Assert(oz_isVar(*v));
+  OzVariable *ov = tagged2Var(*v);
+  switch (ov->getType()) {
+  case OZ_VAR_FAILED:
+    // Don't suspend on a failed value!  addSusp() raises the exception.
+    return ((Failed *) ov)->addSusp(v, susp);
+  case OZ_VAR_EXT:
+    return var2ExtVar(ov)->addSuspV(v, susp);
+  case OZ_VAR_OPT:
+    ov = new SimpleVar(ov->getBoardInternal());
+    *v = makeTaggedVar(ov);
+    // fall through;
+  case OZ_VAR_READONLY_QUIET:
+  case OZ_VAR_READONLY:
+  case OZ_VAR_SIMPLE_QUIET:
+  case OZ_VAR_SIMPLE:
+    if (ozconf.useFutures || susp->isNoBlock()) {
+      return oz_raise(E_ERROR, E_KERNEL, "block", 1, makeTaggedRef(v));
+    }
+    // fall through
+  default:
+    ov->addSuspSVar(susp);
+    return (SUSPEND);
+  }
+}
+
+// raph: makes a variable needed
+OZ_Return oz_var_makeNeeded(TaggedRef *v) {
+  Assert(oz_isVar(*v));
+  OzVariable *ov = tagged2Var(*v);
+
+  switch (ov->getType()) {
+  case OZ_VAR_OPT:
+    ov = new SimpleVar(ov->getBoardInternal());
+    *v = makeTaggedVar(ov);
+    // fall through
+  case OZ_VAR_SIMPLE_QUIET:
+    ((SimpleVar*) ov)->becomeNeeded(); break;
+  case OZ_VAR_READONLY_QUIET:
+    ((ReadOnly*) ov)->becomeNeeded(); break;
+  default:
+    break;
+  }
+
+  return (PROCEED);
+}
+
 void oz_var_dispose(OzVariable *ov) {
   switch (ov->getType()) {
   case OZ_VAR_OPT:     ((OptVar *) ov)->dispose(); break; // debugging, if any;
-  case OZ_VAR_SIMPLE:  ((SimpleVar *) ov)->dispose(); break;
-  case OZ_VAR_FUTURE:  ((Future *) ov)->dispose(); break;
-  case OZ_VAR_BOOL:    ((OzBoolVariable*) ov)->dispose(); break;
-  case OZ_VAR_FD:      ((OzFDVariable*) ov)->dispose(); break;
-  case OZ_VAR_OF:      ((OzOFVariable*) ov)->dispose(); break;
-  case OZ_VAR_FS:      ((OzFSVariable*) ov)->dispose(); break;
-  case OZ_VAR_CT:      ((OzCtVariable*) ov)->dispose(); break;
-  case OZ_VAR_EXT:     var2ExtVar(ov)->disposeV(); break;
+  case OZ_VAR_SIMPLE_QUIET:
+  case OZ_VAR_SIMPLE:   ((SimpleVar *) ov)->dispose(); break;
+  case OZ_VAR_READONLY_QUIET:
+  case OZ_VAR_READONLY: ((ReadOnly *) ov)->dispose(); break;
+  case OZ_VAR_FAILED:   ((Failed *) ov)->dispose(); break;
+  case OZ_VAR_BOOL:     ((OzBoolVariable*) ov)->dispose(); break;
+  case OZ_VAR_FD:       ((OzFDVariable*) ov)->dispose(); break;
+  case OZ_VAR_OF:       ((OzOFVariable*) ov)->dispose(); break;
+  case OZ_VAR_FS:       ((OzFSVariable*) ov)->dispose(); break;
+  case OZ_VAR_CT:       ((OzCtVariable*) ov)->dispose(); break;
+  case OZ_VAR_EXT:      var2ExtVar(ov)->disposeV(); break;
   ExhaustiveSwitch();
   }
 }
@@ -186,12 +269,17 @@ void oz_var_printStream(ostream &out, const char *s, OzVariable *cv, int depth)
     case OZ_VAR_OPT:
       out << s;
       ((OptVar *) cv)->printStream(out, depth); return;
+    case OZ_VAR_SIMPLE_QUIET:
     case OZ_VAR_SIMPLE:
       out << s;
       ((SimpleVar *)cv)->printStream(out,depth); return;
-    case OZ_VAR_FUTURE:
+    case OZ_VAR_READONLY_QUIET:
+    case OZ_VAR_READONLY:
       out << s;
-      ((Future *)cv)->printStream(out,depth); return;
+      ((ReadOnly *)cv)->printStream(out,depth); return;
+    case OZ_VAR_FAILED:
+      out << s;
+      ((Failed *)cv)->printStream(out,depth); return;
     case OZ_VAR_BOOL:
       out << s;
       ((OzBoolVariable*)cv)->printStream(out,depth); return;
@@ -269,40 +357,50 @@ void oz_var_restoreFromCopy(OzVariable * o, OzVariable * c) {
   }  
 }
 
-extern void oz_forceWakeUp(SuspList **);
-
 /*
  * This is the definitive casting table
  *
- *  L=    R=| OPT   | SI    | FU    | EX    | OF    | CT    | FS    | BOOL  | FD    |
- *  ---------------------------------------------------------------------------------
- *  OPT     | NOOP  | NOOP  | NOOP  | NOOP  | NOOP  | NOOP  | NOOP  | NOOP  | NOOP  |
- *  ---------------------------------------------------------------------------------
- *  SI      | NOOP  | NOOP  | NOOP  | NOOP  | NOOP  | NOOP  | NOOP  | NOOP  | NOOP  |
- *  ---------------------------------------------------------------------------------
- *  FU      | NOOP  | NOOP  | NOOP  | NOOP  | NOOP  | NOOP  | NOOP  | NOOP  | NOOP  |
- *  ---------------------------------------------------------------------------------
- *  EX      | NOOP  | NOOP  | NOOP  | NOOP  | NOOP  | NOOP  | NOOP  | NOOP  | NOOP  |
- *  ---------------------------------------------------------------------------------
- *  OF      | R->OF | R->OF | SUSP  | SUSP  | NOOP  | CLASH | CLASH | CLASH | CLASH |
- *  ---------------------------------------------------------------------------------
- *  CT      | R->CT | R->CT | SUSP  | SUSP  | CLASH | NOOP  | CLASH | CLASH | CLASH |
- *  ---------------------------------------------------------------------------------
- *  FS      | R->FS | R->FS | SUSP  | SUSP  | CLASH | CLASH | NOOP  | CLASH | CLASH |
- *  ---------------------------------------------------------------------------------
- *  BOOL    | R->BO | R->BO | SUSP  | SUSP  | CLASH | CLASH | CLASH | NOOP  | NOOP  |
- *  ---------------------------------------------------------------------------------
- *  FD      | R->FD | R->FD | SUSP  | SUSP  | CLASH | CLASH | CLASH | NOOP  | NOOP  |
- *  ---------------------------------------------------------------------------------
+ * (SIQU=SIMPLE_QUIET, ROQU=READONLY_QUIET, RO=READONLY, FAIL=FAILED)
  *
- * Comments (kost@):
+ * L=  R=| OPT   | SIQU  | SI    | EXT   | ROQU  | RO    | FAIL  | OF    | CT    | FS    | BOOL  | FD    |
+ * -------------------------------------------------------------------------------------------------------
+ * OPT   | NOOP  | NOOP  | NOOP  | NOOP  | NOOP  | NOOP  | NOOP  | NOOP  | NOOP  | NOOP  | NOOP  | NOOP  |
+ * -------------------------------------------------------------------------------------------------------
+ * SIQU  | NOOP  | NOOP  | NOOP  | NOOP  | NOOP  | NOOP  | NOOP  | NOOP  | NOOP  | NOOP  | NOOP  | NOOP  |
+ * -------------------------------------------------------------------------------------------------------
+ * SI    | R->SI | R->SI | NOOP  | NOOP  | R->RO | NOOP  | NOOP  | NOOP  | NOOP  | NOOP  | NOOP  | NOOP  |
+ * -------------------------------------------------------------------------------------------------------
+ * EXT   | R->SI | R->SI | NOOP  | NOOP  | R->RO | NOOP  | NOOP  | NOOP  | NOOP  | NOOP  | NOOP  | NOOP  |
+ * -------------------------------------------------------------------------------------------------------
+ * ROQU  | NOOP  | NOOP  | NOOP  | NOOP  | R->RO | NOOP  | NOOP  | NOOP  | NOOP  | NOOP  | NOOP  | NOOP  |
+ * -------------------------------------------------------------------------------------------------------
+ * RO    | R->SI | R->SI | NOOP  | NOOP  | R->RO | NOOP  | NOOP  | NOOP  | NOOP  | NOOP  | NOOP  | NOOP  |
+ * -------------------------------------------------------------------------------------------------------
+ * FAIL  | SUSP  | SUSP  | SUSP  | SUSP  | SUSP  | SUSP  | NOOP  | SUSP  | SUSP  | SUSP  | SUSP  | SUSP  |
+ * -------------------------------------------------------------------------------------------------------
+ * OF    | R->OF | R->OF | R->OF | SUSP  | SUSP  | SUSP  | NOOP  | NOOP  | CLASH | CLASH | CLASH | CLASH |
+ * -------------------------------------------------------------------------------------------------------
+ * CT    | R->CT | R->CT | R->CT | SUSP  | SUSP  | SUSP  | NOOP  | CLASH | NOOP  | CLASH | CLASH | CLASH |
+ * -------------------------------------------------------------------------------------------------------
+ * FS    | R->FS | R->FS | R->FS | SUSP  | SUSP  | SUSP  | NOOP  | CLASH | CLASH | NOOP  | CLASH | CLASH |
+ * -------------------------------------------------------------------------------------------------------
+ * BOOL  | R->BO | R->BO | R->BO | SUSP  | SUSP  | SUSP  | NOOP  | CLASH | CLASH | CLASH | NOOP  | NOOP  |
+ * -------------------------------------------------------------------------------------------------------
+ * FD    | R->FD | R->FD | R->FD | SUSP  | SUSP  | SUSP  | NOOP  | CLASH | CLASH | CLASH | NOOP  | NOOP  |
+ * -------------------------------------------------------------------------------------------------------
+ *
+ * Comments (kost@, raph+fsp):
+ * -  The transitions R->SI and R->RO actually reflect the fact that the
+ *    right-hand side becomes needed.
  * -  Ext -> *      is ignored here: that's the business of that ext var itself!
  *                  Notably, the distribution one;
- * -  Future -> *   is, and has to be, handled by the Future::bind. Don't return 
- *                  'SUSPEND' here!
- * -  Ct -> Future  cannot proceed: unification of Ct"s require a Ct or a non-variable
+ * -  ReadOnly -> * is, and has to be, handled by the ReadOnly::bind.
+ *                  Don't return 'SUSPEND' here!
+ * -  Ct ->ReadOnly cannot proceed: unification of Ct"s require a Ct or a non-variable
  *                  at the right hand side, which is not possible due to the nature.
  * -  Ct -> Ext     cannot proceed either: don't know how to make a Ct out of it!
+ * -  Ct -> Failed  currently the unification fails
+ * -  Failed -> Var currently not implemented (the failed value raises its exception).
  *
  */
 
@@ -316,7 +414,7 @@ OZ_Return oz_var_cast(TaggedRef * & fp, Board * fb, TypeOfVariable tt) {
   TypeOfVariable ft = fv->getType();
 
   OzVariable * tv;
-  
+
   switch (VARTP(tt,ft)) {
 
   case VTP(FD,FS):   case VTP(FD,OF):   case VTP(FD,CT):
@@ -326,33 +424,46 @@ OZ_Return oz_var_cast(TaggedRef * & fp, Board * fb, TypeOfVariable tt) {
   case VTP(OF,FD): case VTP(OF,BOOL): case VTP(OF,FS): case VTP(OF,CT):
     return FAILED;
 
-  case VTP(FD,OPT): case VTP(FD,SIMPLE):
-  case VTP(FD,FUTURE): case VTP(FD,EXT):
+  case VTP(FD,OPT): case VTP(FD,SIMPLE_QUIET): case VTP(FD,SIMPLE):
+  case VTP(FD,READONLY_QUIET): case VTP(FD,READONLY): case VTP(FD,EXT):
     tv = new OzFDVariable(fb);
     break;
 
-  case VTP(BOOL,OPT): case VTP(BOOL,SIMPLE):
-  case VTP(BOOL,FUTURE): case VTP(BOOL,EXT):
+  case VTP(BOOL,OPT): case VTP(BOOL,SIMPLE_QUIET): case VTP(BOOL,SIMPLE):
+  case VTP(BOOL,READONLY_QUIET): case VTP(BOOL,READONLY):
+  case VTP(BOOL,EXT):
     tv = new OzBoolVariable(fb);
     break;
 
-  case VTP(FS,OPT): case VTP(FS,SIMPLE):
-  case VTP(FS,FUTURE): case VTP(FS,EXT):
+  case VTP(FS,OPT): case VTP(FS,SIMPLE_QUIET): case VTP(FS,SIMPLE):
+  case VTP(FS,READONLY_QUIET): case VTP(FS,READONLY):
+  case VTP(FS,EXT):
     tv = new OzFSVariable(fb);
     break;
 
-  case VTP(CT,OPT): case VTP(CT,SIMPLE):
-  case VTP(CT,FUTURE): case VTP(CT,EXT):
+  case VTP(CT,OPT): case VTP(CT,SIMPLE_QUIET): case VTP(CT,SIMPLE):
+  case VTP(CT,READONLY_QUIET): case VTP(CT,READONLY): case VTP(CT,EXT):
     tv = new OzCtVariable(((OzCtVariable *) fv)->getConstraint(), 
 			  ((OzCtVariable *) fv)->getDefinition(), 
 			  fb);
     break;
 
-  case VTP(OF,OPT): case VTP(OF,SIMPLE):
-  case VTP(OF,FUTURE): case VTP(OF,EXT):
+  case VTP(OF,OPT): case VTP(OF,SIMPLE_QUIET): case VTP(OF,SIMPLE):
+  case VTP(OF,READONLY_QUIET): case VTP(OF,READONLY): case VTP(OF,EXT):
     tv = new OzOFVariable(fb);
     break;
 
+  case VTP(SIMPLE,OPT): case VTP(SIMPLE,SIMPLE_QUIET):
+  case VTP(SIMPLE,READONLY_QUIET):
+  case VTP(EXT,OPT): case VTP(EXT,SIMPLE_QUIET):
+  case VTP(EXT,READONLY_QUIET):
+  case VTP(READONLY_QUIET,READONLY_QUIET):
+  case VTP(READONLY,OPT): case VTP(READONLY,SIMPLE_QUIET):
+  case VTP(READONLY,READONLY_QUIET):
+    // makes the right-hand side needed
+    // Note: The unification of readonlys is a demanding operation
+    return oz_var_makeNeeded(fp);
+
   default:
     return PROCEED;
   }
